package net.astah.stpa.stamp.edit;

import static net.astah.project.ProjectPackage.Literals.FACET__CONTENTS;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.uml2.uml.Model;

import net.astah.emf.common.ecore.EcoreCopier;
import net.astah.project.AbstractFacetActivator;
import net.astah.project.Facet;
import net.astah.project.Project;
import net.astah.project.ProjectPackage;
import net.astah.stpa.stamp.HCFHintWordSet;
import net.astah.stpa.stamp.STAMPFactory;
import net.astah.stpa.stamp.STAMPPackage;
import net.astah.stpa.stamp.STPAAnalysis;
import net.astah.stpa.stamp.UCAGuideWord;
import net.astah.stpa.stamp.UCAGuideWordSet;
import net.astah.stpa.stamp.resource.STAMPResources;

/**
 * STAMP/STPA のプロジェクトファセットを維持管理します。
 */
public class STPAFacetActivator extends AbstractFacetActivator {

    @Override
    public Command activate(final EditingDomain domain, final Project project, final Facet facet) {
        final UCAGuideWordSet guideWordSet = STAMPFactory.eINSTANCE.createUCAGuideWordSet();
        final List<UCAGuideWord> guideWords = new ArrayList<>();
        guideWords.add(createGuideWord("N", "Not Providing"));
        guideWords.add(createGuideWord("P", "Providing causes hazard"));
        guideWords.add(createGuideWord("T", "Too early / Too late"));
        guideWords.add(createGuideWord("D", "Stop too soon / Applying too long"));
        guideWordSet.getGuideWords().addAll(guideWords);

        final HCFHintWordSet hintWordSet = createHintWordSet(project);

        final STPAAnalysis analysis = STAMPFactory.eINSTANCE.createSTPAAnalysis();
        analysis.getUCAGuideWordSets().add(guideWordSet);
        analysis.getHCFHintWordSets().add(hintWordSet);

        final Resource resource = createResource(domain, project, "model.stamp");
        final CompoundCommand command = new CompoundCommand();
        command.append(new AddCommand(domain, resource.getContents(), analysis));
        command.append(AddCommand.create(domain, facet, FACET__CONTENTS, analysis));

        project.eAdapters().add(new AdapterImpl() {
            @Override
            public void notifyChanged(final Notification msg) {
                switch (msg.getFeatureID(Project.class)) {
                case ProjectPackage.PROJECT__CONTENTS:
                    switch (msg.getEventType()) {
                    case Notification.ADD:
                        final Object value = msg.getNewValue();
                        if (value instanceof Model) {
                            analysis.setOwner((Model) value);
                            project.eAdapters().remove(this);
                        }
                        break;
                    default:
                        break;
                    }
                    break;
                default:
                    break;
                }
            }
        });

        return command.unwrap();
    }

    @Override
    public Command migrate(final EditingDomain domain, final Facet facet) {
        return null;
    }

    @Override
    protected boolean isFacetResource(final Resource resource) {
        return resource.getContents().stream().anyMatch(this::isFacetContent);
    }

    private boolean isFacetContent(final EObject eObject) {
        return eObject.eClass().getEPackage() == STAMPPackage.eINSTANCE;
    }

    private UCAGuideWord createGuideWord(final String id, final String name) {
        final UCAGuideWord guideWord = STAMPFactory.eINSTANCE.createUCAGuideWord();
        guideWord.setUcaIdPostfix(id);
        guideWord.setName(name);
        return guideWord;
    }

    private HCFHintWordSet createHintWordSet(final Project project) {
        HCFHintWordSet primer = STAMPResources.getLibraryHCFHintWordSet(project,
                "Primer" + "_" + Locale.getDefault().getLanguage());
        if (primer == null || primer.getHintWords().size() == 0) {
            primer = STAMPResources.getLibraryHCFHintWordSet(project, "Primer");
        }
        final EcoreCopier copier = new EcoreCopier();
        final HCFHintWordSet copyObject = (HCFHintWordSet) copier.copy(primer);
        return copyObject;
    }

}
