package net.astah.stpa.stamp.notation.edit.layout;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.domain.EditingDomain;

import com.google.common.collect.Lists;

import net.astah.dd.di.DiagramElement;
import net.astah.emf.edit.command.SortCommand;
import net.astah.notation.edit.layout.NotationDiagramLayoutManager;
import net.astah.notation.provider.NotationZOrderComparator;
import net.astah.stpa.stamp.notation.STAMPDiagram;
import net.astah.stpa.stamp.notation.STAMPDiagramElement;
import net.astah.stpa.stamp.notation.STAMPNotationPackage;

/**
 * STAMP のレイアウト処理を実施するクラスです。
 */
public class STAMPDiagramLayoutManager extends NotationDiagramLayoutManager<STAMPDiagram> {
    public STAMPDiagramLayoutManager() {
        super(STAMPDiagram.class);
    }

    @Override
    public Command processAll(final EditingDomain domain, final STAMPDiagram diagram) {
        Collection<EObject> collection = Lists.newArrayList(diagram.eAllContents()).stream()
                .filter(m -> m instanceof STAMPDiagramElement).collect(Collectors.toList());

        CompoundCommand command = createCommand(domain, collection);
        command.appendIfCanExecute(super.processAll(domain, diagram));
        return command.canExecute() ? command.unwrap() : null;
    }

    @Override
    public Command process(EditingDomain domain, STAMPDiagram diagram, Collection<?> collection,
            List<Notification> notifications) {
        CompoundCommand command = createCommand(domain, collection);
        command.appendIfCanExecute(super.process(domain, diagram, collection, notifications));
        return command.canExecute() ? command.unwrap() : null;
    }

    /**
     * レイアウト処理のためのコマンドを作成します。
     * 
     * @param domain
     *            コマンドを実行するドメイン
     * @param layoutCollection
     *            レイアウト処理対象の図要素
     * @return レイアウト処理のためのコマンド
     */
    protected CompoundCommand createCommand(EditingDomain domain, Collection<?> layoutCollection) {
        CompoundCommand returnCommand = new CompoundCommand();

        Set<Object> sortCollection = new HashSet<Object>(layoutCollection);
        for (Object obj : layoutCollection) {
            if (!(obj instanceof DiagramElement)) {
                continue;
            }

            DiagramElement owningElement = ((DiagramElement) obj).getOwningElement();
            while (owningElement != null) {
                sortCollection.add(owningElement);
                owningElement = owningElement.getOwningElement();
            }
        }

        for (Object obj : sortCollection) {
            if (!(obj instanceof DiagramElement)) {
                continue;
            }

            DiagramElement element = (DiagramElement) obj;
            Command command = SortCommand.create(domain, element,
                    STAMPNotationPackage.Literals.STAMP_DIAGRAM_ELEMENT__STAMP_OWNED_ELEMENT,
                    NotationZOrderComparator.INSTANCE);
            returnCommand.appendIfCanExecute(command);

        }

        return returnCommand;
    }
}
