package net.astah.stpa.stamp.notation.edit.reactor;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.edit.domain.EditingDomain;

import net.astah.dd.di.util.DiagramElements;
import net.astah.emf.common.notify.IPathNotification;
import net.astah.emf.edit.command.DestroyCommand;
import net.astah.emf.edit.notify.EFeaturePathReactor;
import net.astah.stpa.stamp.Component;
import net.astah.stpa.stamp.ProcessModel;
import net.astah.stpa.stamp.notation.STAMPCompartment;
import net.astah.stpa.stamp.notation.STAMPComponentShape;

/**
 * コンポーネントからプロセスモデルが削除された時に図要素の同期処理を行うクラスです。
 */
public class ComponentSynchronizer implements EFeaturePathReactor {
    /**
     * @see EFeaturePathReactor#process(EditingDomain, Collection)
     */
    @Override
    public Command process(EditingDomain domain, Collection<IPathNotification> notifications) {
        final Set<Object> processed = new HashSet<>();
        final CompoundCommand result = new CompoundCommand();

        for (final IPathNotification notification : notifications) {
            final Object notifier = notification.getNotifier();
            if (processed.add(notifier)) {
                final STAMPComponentShape shape = (STAMPComponentShape) notifier;
                final ProcessModel processModel = DiagramElements
                        .getModelElement(shape, Component.class).getProcessModel();
                if (processModel != null) {
                    continue;
                }
                // 全ての区画はドメインモデルを持つ、ドメインモデルを持たない区画はプロセスモデル削除対象の区画と仮定している
                final STAMPCompartment compartment = getCompartmentWithoutModel(
                        shape.getCompartments());
                if (compartment != null) {
                    result.appendIfCanExecute(DestroyCommand.create(domain, compartment));
                }
            }
        }
        return result.isEmpty() ? null : result.unwrap();
    }

    private STAMPCompartment getCompartmentWithoutModel(
            final EList<STAMPCompartment> compartments) {
        return compartments.stream().filter(c -> c.getModelElements().isEmpty()).findFirst()
                .orElse(null);
    }
}
