package net.astah.stpa.stamp.notation.edit.reactor;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.domain.EditingDomain;

import net.astah.emf.common.notify.IPathNotification;
import net.astah.emf.edit.notify.EFeaturePathReactor;
import net.astah.stpa.stamp.notation.STAMPLabel;
import net.astah.stpa.stamp.notation.edit.command.SynchronizeLabelCommand;

/**
 * ラベル図要素の表記の同期処理を行うクラスです。
 */
public class LabelSynchronizer implements EFeaturePathReactor {

    /**
     * @see EFeaturePathReactor#process(EditingDomain, Collection)
     */
    @Override
    public Command process(final EditingDomain domain,
            final Collection<IPathNotification> notifications) {
        final Map<Object, IPathNotification> processed = new HashMap<>();
        final CompoundCommand result = new CompoundCommand();
        for (final IPathNotification notification : notifications) {
            final Object notifier = notification.getNotifier();
            if (notifier instanceof STAMPLabel
                    && processed.putIfAbsent(notifier, notification) == null) {
                result.appendIfCanExecute(
                        new SynchronizeLabelCommand(domain, (STAMPLabel) notifier));
            }
        }
        return result.isEmpty() ? null : result.unwrap();
    }

}
