package net.astah.stpa.stamp.notation.provider;

import static net.astah.stpa.stamp.STAMPPackage.COMPONENT;
import static net.astah.stpa.stamp.STAMPPackage.LINK;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.ACCIDENT_HAZARD_SAFETY_CONSTRAINT_TABLE;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.COMPONENT_EXTRACTING_TABLE;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.COUNTERMEASURE_TABLE;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.HCF_SUMMERY_TABLE;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.HCF_TABLE;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.PRECONDITION_TABLE;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.STAMP_CONTROL_LOOP_DIAGRAM;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.STAMP_CONTROL_STRUCTURE_DIAGRAM;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.UCA_TABLE;

import java.util.Comparator;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

import net.astah.stpa.stamp.STAMPPackage;
import net.astah.stpa.stamp.notation.STAMPNotationPackage;

/**
 * STAMP モデルのツリー内の順序付けを行う {@link Comparator} です
 */
public class STAMPNotationTreeComparator implements Comparator<Object> {

    /**
     * 本クラスのインスタンス
     */
    public static STAMPNotationTreeComparator INSTANCE = new STAMPNotationTreeComparator();

    enum TreeObject {
        COMPONENT_EXTRACTION_TABLE,
        PRECONDITION_TABLE,
        ACCIDENT_HAZARD_SAFETY_CONSTRAINT_TABLE,
        CONTROL_STRUCTURE_DIAGRAM,
        UCA_TABLE,
        CONTROL_LOOP_DIAGRAM,
        HCF_TABLE,
        HCF_SUMMERY_TABLE,
        COUNTERMEASURE_TABLE,
        COMPONENT,
        LINK,
        OTHER,
    }

    @Override
    public int compare(final Object o1, final Object o2) {
        if ((o1 != o2) && o1 instanceof EObject && o2 instanceof EObject) {
            return objectType((EObject) o1).compareTo(objectType((EObject) o2));
        }
        return 0;
    }

    private static TreeObject objectType(final EObject element) {
        final EClass eClass = element.eClass();
        // 図や表、図要素の追加があった場合は、列挙型TreeObjectで構造ツリーでの表示順を定義し、このメソッドのcase文を追加すること
        if (eClass.getEPackage() == STAMPNotationPackage.eINSTANCE) {
            switch (eClass.getClassifierID()) {
            case COMPONENT_EXTRACTING_TABLE:
                return TreeObject.COMPONENT_EXTRACTION_TABLE;
            case PRECONDITION_TABLE:
                return TreeObject.PRECONDITION_TABLE;
            case ACCIDENT_HAZARD_SAFETY_CONSTRAINT_TABLE:
                return TreeObject.ACCIDENT_HAZARD_SAFETY_CONSTRAINT_TABLE;
            case STAMP_CONTROL_STRUCTURE_DIAGRAM:
                return TreeObject.CONTROL_STRUCTURE_DIAGRAM;
            case UCA_TABLE:
                return TreeObject.UCA_TABLE;
            case STAMP_CONTROL_LOOP_DIAGRAM:
                return TreeObject.CONTROL_LOOP_DIAGRAM;
            case HCF_TABLE:
                return TreeObject.HCF_TABLE;
            case HCF_SUMMERY_TABLE:
                return TreeObject.HCF_SUMMERY_TABLE;
            case COUNTERMEASURE_TABLE:
                return TreeObject.COUNTERMEASURE_TABLE;
            default:
            }
        }
        if (eClass.getEPackage() == STAMPPackage.eINSTANCE) {
            switch (eClass.getClassifierID()) {
            case COMPONENT:
                return TreeObject.COMPONENT;
            case LINK:
                return TreeObject.LINK;
            default:
            }
        }
        return TreeObject.OTHER;
    }

}
