package net.astah.stpa.stamp.notation.editor;

import java.util.Collections;
import java.util.Set;

import com.google.common.collect.Iterables;

import lombok.NonNull;
import net.astah.dd.di.DiagramElement;
import net.astah.dd.di.edit.provider.LayoutCalculator;
import net.astah.golf.draw.GNode;
import net.astah.golf.draw.GRimmedGeometry;
import net.astah.golf.geom2D.Pnt2d;
import net.astah.notation.editor.EdgeViewElementImpl;
import net.astah.notation.editor.IViewElement;
import net.astah.notation.editor.ShapeTransformer;
import net.astah.notation.editor.ViewElements;

/**
 * コンポーネント要素の移動・リサイズを実装するクラスです。
 */
public class STAMPComponentTransformer extends ShapeTransformer {

    /** 新しい親ビューエレメント */
    protected IViewElement newParent;

    public STAMPComponentTransformer(@NonNull final STAMPComponentViewElement target,
            @NonNull final Pnt2d startLocation) {
        super(target, startLocation);
        newParent = null;
    }

    public STAMPComponentTransformer(@NonNull final STAMPComponentViewElement target,
            @NonNull final Pnt2d startLocation, int knob) {
        super(target, startLocation, knob);
        newParent = null;
    }

    @Override
    public void dispose() {
        super.dispose();
        newParent = null;
    }

    @Override
    public GNode createFeedbackNode(LayoutCalculator layoutCalculator) {
        createAffectingTransformers();

        final GNode feedbackNode = super.createFeedbackNode(layoutCalculator);

        if (feedbackNode != null && newParent instanceof STAMPComponentViewElement) {
            final GNode node = newParent.createFeedbackShape();
            if (node != null) {
                node.setDepth(-1);
                ((GRimmedGeometry) node).setRimWidth(2);
                feedbackNode.addChild(node);
            }
        }
        return feedbackNode;
    }

    @Override
    public void move(@NonNull final Pnt2d endLocation,
            @NonNull final LayoutCalculator layoutCalculator, final GNode hoverdNode,
            @NonNull final Set<?> constraints) {
        newParent = null;
        super.move(endLocation, layoutCalculator, hoverdNode, constraints);
    }

    @Override
    public STAMPComponentViewElement getTarget() {
        return (STAMPComponentViewElement) super.getTarget();
    }

    @Override
    protected Iterable<IViewElement> getAffectingViewElements() {
        final Iterable<IViewElement> affectings = super.getAffectingViewElements();
        if (newParent != null && getTarget().getParent() != newParent) {
            if (newParent instanceof STAMPComponentViewElement) {
                if (ViewElements.isAncestor((GNode) newParent, (GNode) getTarget())) {
                    return Iterables.concat(affectings, Collections.singleton(newParent),
                            newParent.getAffectingViewElements());
                }
            }
        }
        return affectings;
    }

    @Override
    protected boolean isAffectingElement(IViewElement viewElement) {
        if (newParent != null && getTarget().getParent() != newParent) {
            if (newParent instanceof STAMPComponentViewElement) {
                return (viewElement != getTarget().getParent());
            } else {
                // 図直下
                return (viewElement instanceof EdgeViewElementImpl);
            }
        }
        return super.isAffectingElement(viewElement);
    }

    @Override
    protected DiagramElement getLocationElement(final GNode hoveredNode) {
        if (!isMoveTransform()) {
            return getTarget().getModel().getOwningElement();
        }

        if (hoveredNode == null) {
            newParent = getDiagramViewElement();
            return (DiagramElement) newParent.getModel();
        }

        final STAMPComponentViewElement viewElement = ViewElements.getAncestor(hoveredNode,
                STAMPComponentViewElement.class);
        if (viewElement == null || viewElement.getModel() == null) {
            newParent = getDiagramViewElement();
            return (DiagramElement) newParent.getModel();
        }

        newParent = viewElement;
        return viewElement.getModel();
    }

}
