package net.astah.stpa.stamp.notation.editor;

import javax.swing.ImageIcon;

import net.astah.dd.dc.Bounds;
import net.astah.golf.draw.GGeometry;
import net.astah.golf.draw.GNode;
import net.astah.golf.draw.GOval;
import net.astah.golf.geom2D.Pnt2d;
import net.astah.golf.geom2D.Rectangle2d;
import net.astah.golf.widget.CImageIcon;
import net.astah.golf.widget.swing.SwingIconRegistry;
import net.astah.golf.widget.swing.SwingImageIcon;
import net.astah.jomt.jview.OvalDrawSuggest;
import net.astah.notation.editor.GraphicalStyles;
import net.astah.notation.editor.IDiagramViewElement;
import net.astah.notation.editor.IMovableViewElement;
import net.astah.notation.editor.ITransformer;
import net.astah.notation.editor.ShapeTransformer;
import net.astah.notation.editor.ViewElements;
import net.astah.notation.editor.adjuster.MovingAdjuster;
import net.astah.stpa.stamp.notation.STAMPShape;
import net.astah.stpa.stamp.provider.STAMPEditPlugin;

/**
 * 始点・終点要素を表示するためのビューエレメントを実装するクラスです。
 */
public class STAMPLinkEndViewElement extends STAMPShapeViewElementImpl<STAMPShape>
        implements IMovableViewElement {

    /** 本体ノード */
    protected GOval bodyRect;

    private OvalDrawSuggest drawSuggest;
    private MovingAdjuster.Factory movingAdjusterFactory;

    private static final byte FRAME_COLOR = BLACK;
    private static final byte BODY_COLOR = BLACK;
    private static final byte KNOB_COLOR = BLACK;
    private static final byte SHADOW_COLOR = LIGHTGRAY;

    public STAMPLinkEndViewElement(STAMPShape model) {
        super(model);
    }

    @Override
    public void refresh() {
        super.refresh();
        init();
    }

    /**
     * 初期化処理を行います。
     */
    protected void init() {
        initBodyRect();
        initDrawSuggest();
        initAdjusterFactory();
    }

    @Override
    public <T> T getAdapter(Class<T> adapter) {
        if (adapter.isInstance(drawSuggest)) {
            return adapter.cast(drawSuggest);
        } else if (adapter.isInstance(movingAdjusterFactory)) {
            return adapter.cast(movingAdjusterFactory);
        }
        return super.getAdapter(adapter);
    }

    /**
     * 本体ノードの初期化処理を行います。
     */
    protected void initBodyRect() {
        if (bodyRect == null) {
            int type = 0;
            type = type | RECT; // 2
            type = type | BODY; // 0
            bodyRect = new GOval();
            bodyRect.setType(type);
            bodyRect.setFilled(true);
            bodyRect.setModel(getModel());
            bodyRect.addNodeSelectionListener(bodyRect);
            bodyRect.setSelectionType((byte) (GNode.KNOB | GNode.SHADOW));
            addNodeSelectionListener(bodyRect);
            addNode(bodyRect);
        }
    }

    /**
     * アジャスターファクトリーの初期化処理を行います。
     */
    protected void initAdjusterFactory() {
        movingAdjusterFactory = ViewElements.getAncestor(this, IDiagramViewElement.class)
                .getAdapter(MovingAdjuster.Factory.class);
    }

    @Override
    public void update() {
        super.update();
        updateBodyRect();
    }

    /**
     * 本体ノードの更新処理を行います。
     */
    protected void updateBodyRect() {
        bodyRect.setVisible(true);
        bodyRect.setPickable(true);
        bodyRect.setSelectable(true);
    }

    @Override
    protected void updateStroke() {
        GraphicalStyles.applyStrokeWidth(bodyRect, getModel(), 2);
    }

    @Override
    protected void updateColor() {
        GraphicalStyles.applyFillColor(bodyRect, getModel(), BODY_COLOR);
        GraphicalStyles.applyStrokeColor(bodyRect, getModel(), FRAME_COLOR);

        bodyRect.setKnobColorNo(KNOB_COLOR);
        bodyRect.setShadowColorNo(SHADOW_COLOR);
    }

    @Override
    protected void layout(Bounds bounds) {
        layoutBodyRect(bounds);
    }

    /**
     * 本体ノードのレイアウト計算を行います。
     * 
     * @param bounds
     *            バウンズ
     */
    protected void layoutBodyRect(Bounds bounds) {
        bodyRect.setPoint(new Pnt2d(bounds.getX(), bounds.getY()));
        bodyRect.setWidth(bounds.getWidth());
        bodyRect.setHeight(bounds.getHeight());
    }

    @Override
    protected GGeometry getBounds(Rectangle2d bounds) {
        if (bodyRect != null) {
            bounds.x = bodyRect.getX();
            bounds.y = bodyRect.getY();
            bounds.width = bodyRect.getWidth();
            bounds.height = bodyRect.getHeight();
        }
        return bodyRect;
    }

    @Override
    public void setSelected() {
        bodyRect.setSelected(true);
    }

    @Override
    public ITransformer startMove(Pnt2d startLocation) {
        return new ShapeTransformer(this, startLocation);
    }

    private void initDrawSuggest() {
        drawSuggest = new STAMPLinkEndDrawSuggest(this);
        String key = "diagramview.button.drop_down_component";
        CImageIcon image = new SwingImageIcon((ImageIcon) SwingIconRegistry.INSTANCE
                .getIcon(STAMPEditPlugin.getPlugin().getImage("full/obj16/Link")));
        drawSuggest.addModeAction("net.astah.stpa.stamp.ui.control.mode.CreateControlLink", image,
                key);
    }
}
