package net.astah.stpa.stamp.ui.control;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.transaction.TransactionalEditingDomain;

import net.astah.dd.di.Diagram;
import net.astah.dd.di.util.DiagramElements;
import net.astah.golf.draw.GCamera;
import net.astah.golf.model.BadTransactionException;
import net.astah.golf.model.IEntityStore;
import net.astah.jmodel.DiagramViewInfo;
import net.astah.jomt.jcontrol.CommandSupport;
import net.astah.jomt.jcontrol.OpenDiagramEditorCommand;
import net.astah.jomt.jsystem.ModelGlobal;
import net.astah.jomt.jview.MessageManager;
import net.astah.stpa.stamp.ControlStructure;
import net.astah.stpa.stamp.notation.ComponentExtractingTable;

/**
 * コントロールストラクチャー図を作成するコマンドです。
 * <P>
 * 主に、テーブルのコンテキストメニューからのアクションイベントによる呼び出しを想定していてます。
 * </P>
 */
public class CreateControlStructureDiagramFromTableCommand extends net.astah.golf.control.Command {
    private ComponentExtractingTable table;

    @Override
    public void execute() {
        if (ModelGlobal.prjManager == null || ModelGlobal.prjManager.getCurrentProject() == null) {
            return;
        }
        IEntityStore doc = ModelGlobal.prjManager.getCurrentProject().doc;
        if (doc == null) {
            return;
        }
        if (table == null) {
            table = getSelectedComponentExtractingTable();
        }
        if (table == null) {
            return;
        }
        if (table.getControlStructureDiagram() != null) {
            openCreatedDiagram(table.getControlStructureDiagram());
            return;
        }

        TransactionalEditingDomain domain = (TransactionalEditingDomain) doc.getDomain();
        try {
            doc.beginTransaction();

            Diagram createdDiagram = createDiagram(domain);
            Command command = new ModifyControlStructureDiagramElementsCommand(domain,
                    (ControlStructure) DiagramElements.getModelElement(createdDiagram));
            if (command.canExecute()) {
                command.execute();
            }

            doc.commitTransaction();

            openCreatedDiagram(createdDiagram);
        } catch (BadTransactionException e) {
            MessageManager.requireBugReport(e);
            doc.abortTransaction();
        } catch (Exception e) {
            MessageManager.requireBugReport(e);
            doc.abortTransaction();
        } finally {
        }
    }

    private ComponentExtractingTable getSelectedComponentExtractingTable() {
        return CommandSupport.instance().getSelectedModelElementFromProjectView();
    }

    private void openCreatedDiagram(Diagram diagram) {
        OpenDiagramEditorCommand cmd = new OpenDiagramEditorCommand();
        DiagramViewInfo viewInfo = new DiagramViewInfo(diagram, new GCamera());
        cmd.setDiagramViewInfo(viewInfo);
        callCommand(cmd);
    }

    private Diagram createDiagram(TransactionalEditingDomain domain) {
        CreateControlStructureDiagramFromPrjCommand cmd = new CreateControlStructureDiagramFromPrjCommand();
        cmd.setUseTransaction(false);
        cmd.setOpenEditorAfterCreate(false);
        callCommand(cmd);
        return cmd.getDiagram();
    }

    public void setComponentExtractingTable(ComponentExtractingTable table) {
        this.table = table;
    }

}