package net.astah.stpa.stamp.ui.control;

import static java.util.Collections.singletonList;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.ResourceSet;

import net.astah.emf.common.ecore.EObjects;
import net.astah.golf.model.IEntityStore;
import net.astah.jomt.jcontrol.CreateDiagramFromPrjCommand;
import net.astah.jomt.jcontrol.CreateNewPrjCommand;
import net.astah.project.Facets;
import net.astah.project.Project;
import net.astah.project.ProjectManagerAdapter;
import net.astah.stpa.stamp.Element;
import net.astah.stpa.stamp.STAMPPlugin;
import net.astah.stpa.stamp.STPAAnalysis;
import net.astah.stpa.stamp.notation.STAMPDiagram;
import net.astah.stpa.stamp.notation.STAMPDiagramElement;
import net.astah.stpa.stamp.notation.STAMPTable;

/**
 * STAMP の図を作成するコマンドの抽象基本クラスです。
 */
public abstract class CreateSTAMPDiagramFromPrjCommand extends CreateDiagramFromPrjCommand {

    @Override
    protected CreateNewPrjCommand createNewProject() {
        final CreateNewPrjCommand newPrjCmd = new CreateNewPrjCommand();
        newPrjCmd.setInitialFacets(singletonList(STAMPPlugin.FACET_SYMBOLIC_NAME));
        return newPrjCmd;
    }

    @Override
    protected EObject getOwnerModel(final IEntityStore doc, final EObject selectedModel) {
        EObject owner = super.getOwnerModel(doc, selectedModel);
        if (owner == null) {
            final ResourceSet resourceSet = doc.getDomain().getResourceSet();
            final Project project = ProjectManagerAdapter.getRootProject(resourceSet);
            owner = Facets.getFirstContent(project, STAMPPlugin.FACET_SYMBOLIC_NAME,
                    STPAAnalysis.class, null);
        }
        return owner;
    }

    @Override
    protected boolean isCorrectModel(final Object model) {
        return model instanceof STPAAnalysis;
    }

    @Override
    protected EObject selectCorrectModel(final EObject incorrectModel) {
        if (incorrectModel instanceof Element) {
            final STPAAnalysis ancestor = EObjects.getAncestor(STPAAnalysis.class, incorrectModel);
            if (ancestor != null) {
                return ancestor;
            }
        } else if (incorrectModel instanceof STAMPDiagramElement) {
            final STAMPDiagram ancestor = EObjects.getAncestor(STAMPDiagram.class, incorrectModel);
            if (ancestor != null) {
                final EObject owner = ancestor.getOwner();
                if (owner != null) {
                    return owner;
                }
            }
        } else if (incorrectModel instanceof STAMPTable) {
            final STAMPTable ancestor = EObjects.getAncestor(STAMPTable.class, incorrectModel);
            if (ancestor != null) {
                final EObject owner = ancestor.getOwner();
                if (owner != null) {
                    return owner;
                }
            }
        }

        final Project project = ProjectManagerAdapter.getRootProject(incorrectModel);
        return Facets.getFirstContent(project, STAMPPlugin.FACET_SYMBOLIC_NAME, STPAAnalysis.class,
                null);
    }

}
