package net.astah.stpa.stamp.ui.control;

import java.beans.PropertyVetoException;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.astah.golf.control.Command;
import net.astah.jomt.jview.DiagramEditor;
import net.astah.project.Facets;
import net.astah.project.Project;
import net.astah.project.ProjectManagerAdapter;
import net.astah.stpa.stamp.ControlLoop;
import net.astah.stpa.stamp.HCFHintWordSet;
import net.astah.stpa.stamp.STAMPPlugin;
import net.astah.stpa.stamp.STPAAnalysis;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.stpa.stamp.ui.view.HCFHintWordSetFrame;
import net.astah.stpa.stamp.util.HCFHintWordSets;

/**
 * ヒントワードセット画像表示パレットフレームを表示または非表示にするコマンドです。
 */
public class OpenHintWordImageCommand extends Command {

    private static final Logger LOGGER = LoggerFactory.getLogger(OpenHintWordImageCommand.class);

    @Override
    public void execute() {
        final ControlLoop controlLoop = HCFHintWordSetFrame.getCurrentControlLoop();
        if (controlLoop == null) {
            return;
        }
        final HCFHintWordSetFrame palette = (HCFHintWordSetFrame) DiagramEditor.getPaletteframe();
        if (palette != null) {
            palette.updateComboBox(controlLoop);
            palette.setVisible(!palette.isVisible());
        } else {
            createHintWordSetForm(controlLoop);
        }
    }

    private void createHintWordSetForm(final ControlLoop controlLoop) {
        // ウィンドウの本体部分の作成
        final HCFHintWordSetFrame palette = createHintWordSetFrame(controlLoop);
        if (palette == null) {
            return;
        }
        final EList<HCFHintWordSet> hintWordSets = getHintWordSets(controlLoop);
        palette.createHintWordSetFrame(hintWordSets, controlLoop);
    }

    private EList<HCFHintWordSet> getHintWordSets(final ControlLoop controlLoop) {
        final Project project = ProjectManagerAdapter.getRootProject(controlLoop);
        final STPAAnalysis analysis = Facets.getFirstContent(project,
                STAMPPlugin.FACET_SYMBOLIC_NAME, STPAAnalysis.class, null);
        return HCFHintWordSets.getLanguageHintWordSets(analysis,
                controlLoop.eResource().getResourceSet());
    }

    private HCFHintWordSetFrame createHintWordSetFrame(final ControlLoop controlLoop) {
        final EditingDomain domain = TransactionUtil.getEditingDomain(controlLoop);

        try {
            final HCFHintWordSetFrame palette = new HCFHintWordSetFrame(domain,
                    Messages.hcf_hintword_set_frame_header, true, true, false, true);
            palette.initializeFrame();
            return palette;
        } catch (final PropertyVetoException e) {
            LOGGER.error("error has occurred.", e);
        }
        return null;
    }

}
