package net.astah.stpa.stamp.ui.control.mode;

import java.awt.event.MouseEvent;

import net.astah.dd.di.DiagramElement;
import net.astah.golf.control.CommandEvent;
import net.astah.golf.draw.GNode;
import net.astah.notation.Edge;
import net.astah.notation.editor.ViewElements;
import net.astah.stpa.stamp.LinkEnd;
import net.astah.stpa.stamp.notation.STAMPComponentShape;
import net.astah.stpa.stamp.notation.STAMPNotationFactory;
import net.astah.stpa.stamp.notation.editor.STAMPLinkEndViewElement;
import net.astah.ui.control.CreateEdgeCommand;
import net.astah.ui.control.mode.CreateRelationMode;

/**
 * リンクを作成する Mode です。
 */
public abstract class CreateLinkMode extends CreateRelationMode {

    @Override
    protected Edge createEdge() {
        return STAMPNotationFactory.eINSTANCE.createSTAMPEdge();
    }

    /**
     * リンクを作成するコマンドを作成して実行します。
     */
    @Override
    public void fireCommand(final MouseEvent e) {
        final DiagramElement sourceDiagramElement = getDiagramElement(source);
        final DiagramElement targetDiagramElement = getDiagramElement(target);
        final CreateEdgeCommand command = getCommand();
        final Object[] edgeFactor = createEdgeFactor();
        command.setEdge((Edge) edgeFactor[0]);
        command.setSourceAnchor(edgeFactor[1]);
        command.setTargetAnchor(edgeFactor[2]);
        command.setLine(edgeFactor[3]);
        command.setShiftPressedState(e.isShiftDown());
        command.setSource(sourceDiagramElement);
        command.setTarget(targetDiagramElement);

        fireCommandPerformed(new CommandEvent(e.getSource(), e.getID(), "CreateLink", command,
                e.getModifiers()));
        e.consume();
    }

    @Override
    protected GNode getAppropriateNode(final GNode pickedNode) {
        final GNode node = ViewElements.getAncestor(pickedNode,
                ancestor -> isValid(getDiagramElement(ancestor)));
        return node;
    }

    @Override
    protected boolean isValidTarget(final DiagramElement diagramElement) {
        if (source instanceof STAMPLinkEndViewElement && hasModel(diagramElement, LinkEnd.class)) {
            return false;
        }
        return isValid(diagramElement);
    }

    @Override
    public boolean isLegalStart(final DiagramElement diagramElement) {
        return isValid(diagramElement);
    }

    @Override
    public boolean isLegalEnd(final DiagramElement diagramElement) {
        if (source instanceof STAMPLinkEndViewElement && hasModel(diagramElement, LinkEnd.class)) {
            return false;
        }
        return isValid(diagramElement);
    }

    private boolean isValid(final DiagramElement diagramElement) {
        return diagramElement instanceof STAMPComponentShape
                || hasModel(diagramElement, LinkEnd.class);
    }
}
