package net.astah.stpa.stamp.ui.menu.handler;

import static java.util.Collections.singleton;
import static net.astah.stpa.stamp.STAMPPackage.Literals.HAZARD_CAUSAL_FACTOR__COUNTERMEASURE;
import static net.astah.stpa.stamp.STAMPPackage.Literals.STPA_ANALYSIS__COUNTERMEASURE;
import static net.astah.stpa.stamp.ui.view.swing.CountermeasureTableEditorPeer.COUNTERMEASURE_COLUMN;
import static net.astah.stpa.stamp.ui.view.swing.CountermeasureTableEditorPeer.COUNTERMEASURE_ID_COLUMN;
import static net.astah.stpa.stamp.ui.view.swing.CountermeasureTableEditorPeer.COUNTERMEASURE_INDEX;
import static net.astah.stpa.stamp.ui.view.swing.CountermeasureTableEditorPeer.HAZARD_CAUSAL_FACTOR_INDEX;
import static net.astah.table.TablePackage.Literals.ELEMENT__OBJECT;
import static net.astah.table.TablePackage.Literals.GRID__ROW_TUPLE;
import static org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain.getEditingDomainFor;

import java.awt.event.ActionEvent;
import java.util.List;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.command.UnexecutableCommand;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.command.CreateChildCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.emf.edit.job.CommandJob;
import net.astah.emf.edit.job.Jobs;
import net.astah.golf.widget.IListSelection;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.ITableSelection;
import net.astah.jomt.jsystem.JomtGlobal;
import net.astah.jomt.jview.Editor;
import net.astah.jomt.jview.EditorPeer;
import net.astah.stpa.stamp.Countermeasure;
import net.astah.stpa.stamp.HazardCausalFactor;
import net.astah.stpa.stamp.STAMPFactory;
import net.astah.stpa.stamp.STPAAnalysis;
import net.astah.stpa.stamp.ui.view.swing.CountermeasureTableEditorPeer;
import net.astah.table.Element;
import net.astah.table.Grid;
import net.astah.table.Tuple;
import net.astah.table.edit.command.AddRowCommand;
import net.astah.table.ui.TableEditors;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 対策表に対策を追加します。
 */
@Handler
@MenuItem(commands = @MenuCommand(id = "stpa.AddCountermeasureCommand",
        category = "edit.table"),
        label = "%StampTable_addCountermeasure_label",
        tooltip = "%StampTable_addCountermeasure_tooltip",
        type = ItemKind.PUSH)
public class AddCountermeasureHandler {

    private Command command = UnexecutableCommand.INSTANCE;

    private int affectedRow = -1;

    /**
     * 対策を追加するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection,
            final List<String> params,
            final ActionEvent e) {
        final CommandJob job = new CommandJob(getDomain(), command);
        Jobs.toCompletableFuture(job, true).thenRun(() -> {
            TableEditors.startCellEditing(affectedRow, COUNTERMEASURE_COLUMN);
        });
        job.schedule();
    }

    private TransactionalEditingDomain getDomain() {
        final Editor editor = JomtGlobal.viewManager.getCurrentEditor();
        return TransactionUtil.getEditingDomain(editor.getEditModel());
    }

    /**
     * 対策を追加するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        command.dispose();
        command = UnexecutableCommand.INSTANCE;
        if (selection instanceof ITableSelection) {
            command = getCommand((ITableSelection) selection);
        }
        return command.canExecute();
    }

    private Command getCommand(final ITableSelection selection) {
        final Object element = selection.getFirstElement();
        if (element instanceof Tuple) {
            if (isCountermeasure(selection.getColumnSelection())) {
                return createCommand((Tuple) element);
            }
        }
        return command;
    }

    private boolean isCountermeasure(final IListSelection columnSelection) {
        final int[] indices = columnSelection.getIndices();
        if (indices.length > 0) {
            return indices[0] == COUNTERMEASURE_ID_COLUMN || indices[0] == COUNTERMEASURE_COLUMN;
        }
        return false;
    }

    /**
     * 選択されている行タプルを起点に対策を追加するコマンドを構築します。
     * 
     * @param tuple
     *            選択されている行タプル
     * @return 構築したコマンド
     */
    public Command createCommand(final Tuple tuple) {
        final int row = tuple.getIndex();
        final Grid grid = tuple.getGrid();
        final EList<Tuple> rowTuples = grid.getRowTuples();
        if ((row >= 0) && (row < rowTuples.size())) {
            final Tuple rowTuple = rowTuples.get(row);
            return createAddCountermeasureCommand(grid, row, rowTuple);
        }
        return command;
    }

    /**
     * 選択されている行タプルを起点に対策を追加するコマンドを構築します。
     * 
     * @param grid
     *            対策を追加するグリッド
     * @param row
     *            選択されている行のインデックス
     * @param rowTuple
     *            選択されている行のタプル
     * @return 構築したコマンド
     */
    protected Command createAddCountermeasureCommand(final Grid grid,
            final int row,
            final Tuple rowTuple) {
        final STPAAnalysis scope = getEditorPeer().getScope();
        final EditingDomain domain = getEditingDomainFor(scope);
        final Countermeasure newValue = STAMPFactory.eINSTANCE.createCountermeasure();
        final EList<Element> elements = rowTuple.getElements();
        final Element valueElement = elements.get(COUNTERMEASURE_INDEX);
        final Countermeasure value = (Countermeasure) valueElement.getObject();
        final Element hcfElement = elements.get(HAZARD_CAUSAL_FACTOR_INDEX);
        final HazardCausalFactor hcf = (HazardCausalFactor) hcfElement.getObject();
        final int index = hcf.getCountermeasures().indexOf(value);
        affectedRow = row + (index < 0 ? 0 : 1);

        final CompoundCommand command = new CompoundCommand();
        command.append(CreateChildCommand.create(domain, scope,
                new CommandParameter(scope, STPA_ANALYSIS__COUNTERMEASURE, newValue),
                singleton(scope)));
        command.append(AddCommand.create(domain, hcf, HAZARD_CAUSAL_FACTOR__COUNTERMEASURE,
                newValue, index + 1));

        if (value == null) {
            // 現在の行に対策が設定されていないので、現在行に対策を追加する
            command.append(SetCommand.create(getDomain(), valueElement, ELEMENT__OBJECT, newValue));
        } else {
            // 現在の行に対策が設定されているので、新規行として対策を追加する
            final Tuple newRowTuple = AddRowCommand.prepareNewTuple(grid, row);
            final EList<Element> newElements = newRowTuple.getElements();
            newElements.get(HAZARD_CAUSAL_FACTOR_INDEX).setObject(hcf);
            newElements.get(COUNTERMEASURE_INDEX).setObject(newValue);
            command.append(AddCommand.create(getDomain(), grid, GRID__ROW_TUPLE, newRowTuple,
                    affectedRow));
        }

        return command;
    }

    private CountermeasureTableEditorPeer getEditorPeer() {
        final Editor editor = JomtGlobal.viewManager.getCurrentEditor();
        final EditorPeer editorPeer = editor.getEditorPeer();
        return (CountermeasureTableEditorPeer) editorPeer;
    }

}
