package net.astah.stpa.stamp.ui.menu.handler;

import static org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain.getEditingDomainFor;

import java.awt.event.ActionEvent;
import java.util.List;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.UnexecutableCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.emf.edit.job.CommandJob;
import net.astah.emf.edit.job.Jobs;
import net.astah.golf.widget.IListSelection;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.ITableSelection;
import net.astah.jomt.jsystem.JomtGlobal;
import net.astah.jomt.jview.Editor;
import net.astah.jomt.jview.EditorPeer;
import net.astah.stpa.stamp.ui.view.swing.HCFTableEditorPeer;
import net.astah.table.Grid;
import net.astah.table.Tuple;
import net.astah.table.edit.command.AddRowCommand;
import net.astah.table.ui.TableEditors;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * HCF表にHCFを追加します。
 *
 */
@Handler
@MenuItem(commands = @MenuCommand(id = "stpa.AddHazardCausalFactorCommand",
        category = "edit.table"),
        label = "%StampTable_addHazardCausalFactor_label",
        tooltip = "%StampTable_addHazardCausalFactor_tooltip",
        type = ItemKind.PUSH)
public class AddHazardCausalFactorHandler {

    private final int HAZARD_CAUSAL_FACTOR__COLUMN = 1;

    private Command command = UnexecutableCommand.INSTANCE;

    /**
     * HCF表にHCFを追加するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final Editor editor = JomtGlobal.viewManager.getCurrentEditor();
        final TransactionalEditingDomain domain = TransactionUtil
                .getEditingDomain(editor.getEditModel());
        final CommandJob job = new CommandJob(domain, command);
        Jobs.toCompletableFuture(job, true).thenRun(() -> {
            TableEditors.startCellEditing(-1, HAZARD_CAUSAL_FACTOR__COLUMN);
        });
        job.schedule();
    }

    /**
     * HCF表にHCFを追加するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        command.dispose();
        command = UnexecutableCommand.INSTANCE;
        if (selection instanceof ITableSelection) {
            command = getCommand((ITableSelection) selection);
        }
        return command.canExecute();
    }

    private Command getCommand(final ITableSelection selection) {
        if (selection.isEmpty()) {
            final Grid grid = getGrid();
            if (grid != null) {
                return createCommand(grid);
            }
        } else {
            final Object element = selection.getFirstElement();
            if (element instanceof Tuple) {
                if (isHazardCausalFactor(selection.getColumnSelection())) {
                    return createCommand((Tuple) element);
                }
            }
        }
        return command;
    }

    private boolean isHazardCausalFactor(final IListSelection columnSelection) {
        final int[] indices = columnSelection.getIndices();
        if (indices.length > 0) {
            return indices[0] <= HAZARD_CAUSAL_FACTOR__COLUMN;
        }
        return false;
    }

    private Command createCommand(final Tuple tuple) {
        final Grid grid = tuple.getGrid();
        final int index = tuple.getIndex();
        final Tuple rowTuple = AddRowCommand.prepareNewTuple(grid, index);
        return AddRowCommand.create(getEditingDomainFor(tuple), grid, rowTuple, index);
    }

    private Command createCommand(final Grid grid) {
        final Tuple rowTuple = AddRowCommand.prepareNewTuple(grid, -1);
        return AddRowCommand.create(getEditingDomainFor(grid), grid, rowTuple, 0);
    }

    private Grid getGrid() {
        final Editor editor = JomtGlobal.viewManager.getCurrentEditor();
        final EditorPeer editorPeer = editor.getEditorPeer();
        return ((HCFTableEditorPeer) editorPeer).getGrid();
    }

}
