package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.golf.control.GolfAction;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jomt.jcontrol.DepthCommand;
import net.astah.stpa.stamp.notation.STAMPDiagram;
import net.astah.stpa.stamp.notation.STAMPEdge;
import net.astah.stpa.stamp.notation.STAMPLabel;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 図要素の表示順序(深さ)を指定します。
 */
@Handler(commands = "stpa.ArrangeDepthCommand")
@MenuItem(commands = @MenuCommand(id = "stpa.ArrangeDepthCommand",
        category = "edit.depth"),
        path = "Depth",
        label = "%StampControlStructureDiagram_depth_label",
        tooltip = "%StampControlStructureDiagram_depth_tooltip",
        type = ItemKind.MENU)
@MenuItem(commands = @MenuCommand(id = "stpa.ArrangeDepthCommand",
        category = "edit.depth",
        parameters = DepthCommand.FRONT_DEPTH),
        path = "Depth." + DepthCommand.FRONT_DEPTH,
        label = "%StampControlStructureDiagram_depth_front_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/depth_front.png",
        tooltip = "%StampControlStructureDiagram_depth_front_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.ArrangeDepthCommand",
        category = "edit.depth",
        parameters = DepthCommand.FORWARD_DEPTH),
        path = "Depth." + DepthCommand.FORWARD_DEPTH,
        label = "%StampControlStructureDiagram_depth_forward_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/depth_forward.png",
        tooltip = "%StampControlStructureDiagram_depth_forward_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.ArrangeDepthCommand",
        category = "edit.depth",
        parameters = DepthCommand.BACKWARD_DEPTH),
        path = "Depth." + DepthCommand.BACKWARD_DEPTH,
        label = "%StampControlStructureDiagram_depth_backward_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/depth_backward.png",
        tooltip = "%StampControlStructureDiagram_depth_backward_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.ArrangeDepthCommand",
        category = "edit.depth",
        parameters = DepthCommand.BACK_DEPTH),
        path = "Depth." + DepthCommand.BACK_DEPTH,
        label = "%StampControlStructureDiagram_depth_back_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/depth_back.png",
        tooltip = "%StampControlStructureDiagram_depth_back_tooltip",
        type = ItemKind.CHECK)
public class ArrangeDepthHandler {

    /**
     * 図要素の表示順序(深さ)を指定するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final GolfAction action = new GolfAction("ArrangeDepth%" + params.get(0));
        action.actionPerformed(e);
    }

    /**
     * 図要素の表示順序(深さ)を指定するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 図、STAMPEdge、STAMPLabelのみの場合、false
        return Selections.stream(selection).anyMatch(object -> isTarget(object));
    }

    private boolean isTarget(final Object object) {
        return !(object instanceof STAMPDiagram
                || object instanceof STAMPEdge
                || object instanceof STAMPLabel);
    }

}
