package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.golf.util.GolfUtilities;
import net.astah.golf.widget.ISelection;
import net.astah.jomt.jcontrol.CopyWithEMFCommand;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.KeyBinding;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 選択した図要素をクリップボードにコピーします。 (EMF)
 * 
 * <p>
 * {@link CopysHandler} が親のメニューを持ち、BMP、PNGのコピーを実行する.
 */
@Handler(commands = "stpa.CopyWithEMFCommand")
@MenuItem(commands = @MenuCommand(id = "stpa.CopyWithEMFCommand",
        category = "edit.copy_paste",
        parameters = "image"),
        keyBinding = @KeyBinding(keySequence = "control shift C"),
        path = "CopyToClipboard." + "emf",
        label = "%StampControlStructureDiagram_copyToEmf_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/copy.png",
        tooltip = "%StampControlStructureDiagram_copyToEmf_tooltip",
        type = ItemKind.CHECK)
public class CopyWithEMFHandler {

    /**
     * 選択した図要素を EMF 形式でクリップボードにコピーするコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final CopyWithEMFCommand command = new CopyWithEMFCommand();
        command.setArgumentString(params.get(0));
        command.setUseTransaction(true);
        command.execute();
    }

    /**
     * 選択した図要素を EMF 形式でクリップボードにコピーするコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false Mac の場合はコマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // Macの場合、false
        return !GolfUtilities.isMacOS();
    }

}
