package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.golf.control.CommandManager;
import net.astah.golf.widget.ISelection;
import net.astah.jomt.jsystem.JomtGlobal;
import net.astah.jomt.jview.Editor;
import net.astah.stpa.stamp.notation.ComponentExtractingTable;
import net.astah.stpa.stamp.ui.control.CreateControlStructureDiagramFromTableCommand;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * コントロール抽出表からコントロールストラクチャー図を生成します。
 */
@Handler
@MenuItem(commands = @MenuCommand(id = "stpa.CreateControlStructureDiagramCommand",
        category = "edit.table"),
        label = "%ComponentExtractingTable_createControlStructureDiagram_label",
        tooltip = "%ComponentExtractingTable_createControlStructureDiagram_tooltip",
        type = ItemKind.PUSH)
public class CreateControlStructureDiagramHandler {

    private ComponentExtractingTable table;

    /**
     * コントロール抽出表からコントロールストラクチャー図を生成するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        CreateControlStructureDiagramFromTableCommand cmd = new CreateControlStructureDiagramFromTableCommand();
        cmd.setComponentExtractingTable(table);
        CommandManager.instance().callCommand(cmd);
    }

    /**
     * コンポーネント抽出表からコントロールストラクチャー図を生成する機能を追加するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        table = getComponentExtractingTable();
        return table != null;
    }

    private ComponentExtractingTable getComponentExtractingTable() {
        if (JomtGlobal.viewManager == null) {
            return null;
        }
        Editor currentEditor = JomtGlobal.viewManager.getCurrentEditor();
        if (currentEditor != null
                && currentEditor.getEditModel() instanceof ComponentExtractingTable) {
            return (ComponentExtractingTable) currentEditor.getEditModel();
        }
        return null;
    }
}
