package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import net.astah.dd.di.util.DiagramElements;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jomt.jcontrol.DeleteFromDgmCommand;
import net.astah.stpa.stamp.LinkEnd;
import net.astah.stpa.stamp.Note;
import net.astah.stpa.stamp.NoteAnchor;
import net.astah.stpa.stamp.notation.STAMPDiagram;
import net.astah.stpa.stamp.notation.STAMPDiagramElement;
import net.astah.stpa.stamp.notation.STAMPLabel;
import net.astah.stpa.stamp.notation.STAMPShape;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.KeyBinding;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 選択した図要素を図から削除します。
 */
@Handler
@MenuItem(commands = @MenuCommand(id = "stpa.DeleteFromDiagramCommand",
        category = "edit.model"),
        keyBinding = @KeyBinding(keySequence = "DELETE"),
        label = "%StampControlStructureDiagram_deleteFromDiagram_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/delDgm.png",
        tooltip = "%StampControlStructureDiagram_deleteFromDiagram_tooltip",
        type = ItemKind.PUSH)
public class DeleteFromDiagramHandler {

    /**
     * 選択した図要素を図から削除するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final DeleteFromDgmCommand command = new DeleteFromDgmCommand();
        command.setDoTransaction(true);
        command.execute();
    }

    /**
     * 選択した図要素を図から削除するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 図、CA、FB、LinkEnd、ノート、ノートアンカー以外の要素があればtrue
        return Selections.stream(selection).anyMatch(object -> isTarget(object));
    }

    private boolean isTarget(final Object object) {
        // 図、STAMPLabel、LinkEndは、false
        if (object instanceof STAMPDiagram
                || object instanceof STAMPLabel
                || isLinkEnd(object)) {
            return false;
        }
        // ノート、ノートアンカー以外はtrue
        if (object instanceof STAMPDiagramElement) {
            final EObject element = DiagramElements.getModelElement((STAMPDiagramElement) object);
            return element == null // テキスト、画像は無し
                    || !(element instanceof Note || element instanceof NoteAnchor);
        }
        return true;
    }

    private boolean isLinkEnd(final Object object) {
        return object instanceof STAMPShape
                && DiagramElements.isModelElementPresent((STAMPShape) object, LinkEnd.class);
    }

}
