package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.di.util.DiagramElements;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jomt.jcontrol.DeleteFromModelCommand;
import net.astah.stpa.stamp.notation.STAMPDiagram;
import net.astah.stpa.stamp.notation.STAMPDiagramElement;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.KeyBinding;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 選択した図要素をモデルから削除します。
 */
@Handler
@MenuItem(label = "%StampControlStructureDiagram_deleteFromModel_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/delMdl.png",
        tooltip = "%StampControlStructureDiagram_deleteFromModel_tooltip",
        type = ItemKind.PUSH)
@MenuCommand(id = "stpa.DeleteFromModelCommand",
        category = "edit.model")
@KeyBinding(keySequence = "control D")
public class DeleteFromModelHandler {

    /**
     * 選択した図要素をモデルから削除するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final DeleteFromModelCommand command = new DeleteFromModelCommand();
        command.setUseTransaction(true);
        command.execute();
    }

    /**
     * 選択した図要素をモデルから削除するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        return Selections.stream(selection).anyMatch(object -> isTarget(object));
    }

    private boolean isTarget(final Object object) {
        // 図、共通図要素以外はtrue
        if (object instanceof STAMPDiagram) {
            return false;
        }
        if (object instanceof STAMPDiagramElement) {
            // テキスト、画像はmodelElement無し
            return DiagramElements.isModelElementPresent((STAMPDiagramElement) object);
        }
        return false;
    }

}
