package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;
import java.util.stream.Collectors;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.UnexecutableCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.dd.di.util.DiagramElements;
import net.astah.emf.edit.command.DestroyCommand;
import net.astah.emf.edit.job.CommandJob;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.stpa.stamp.Component;
import net.astah.stpa.stamp.ProcessModel;
import net.astah.stpa.stamp.notation.STAMPComponentShape;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 図上でプロセスモデルのモデルからの削除します。
 */
@Handler(targets = { STAMPComponentShape.class })
@MenuItem(label = "%StampControlStructureDiagram_deleteProcessModel_label",
        tooltip = "%StampControlStructureDiagram_deleteProcessModel_tooltip",
        type = ItemKind.PUSH)
@MenuCommand(id = "stpa.DeleteProcessModelCommand",
        category = "edit.delete_process_model")
public class DeleteProcessModelHandler {
    private Command command = UnexecutableCommand.INSTANCE;

    /**
     * プロセスモデルをモデルから削除するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final STAMPComponentShape shape = (STAMPComponentShape) Selections
                .getFirstElement(selection);
        final Component component = DiagramElements.getModelElement(shape, Component.class);
        final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(component);
        final CommandJob job = new CommandJob(domain, command);
        job.schedule();
    }

    private Command getCommand(final ISelection selection) {
        if (Selections.size(selection) != 1) {
            return UnexecutableCommand.INSTANCE;
        }
        final STAMPComponentShape shape = (STAMPComponentShape) Selections
                .getFirstElement(selection);
        final Component component = DiagramElements.getModelElement(shape, Component.class);
        final ProcessModel processModel = component.getProcessModel();
        if (processModel == null) {
            return UnexecutableCommand.INSTANCE;
        }
        final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(component);
        final List<Object> collection = shape.getCompartments().stream()
                .filter(compartment -> compartment.getModelElements().contains(processModel))
                .collect(Collectors.toList());
        if (collection.isEmpty()) {
            return UnexecutableCommand.INSTANCE;
        }
        collection.add(processModel);
        return DestroyCommand.create(domain, collection);
    }

    /**
     * プロセスモデルをモデルから削除するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        command.dispose();
        command = getCommand(selection);
        return command.canExecute();
    }
}
