package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.dg.Group;
import net.astah.dd.dg.Rectangle;
import net.astah.dd.dg.Text;
import net.astah.dd.di.util.DiagramElements;
import net.astah.golf.control.GolfAction;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.stpa.stamp.Component;
import net.astah.stpa.stamp.Note;
import net.astah.stpa.stamp.Signal;
import net.astah.stpa.stamp.notation.STAMPShape;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 図要素の文字のフォントを設定します。
 */
@Handler
@MenuItem(label = "%StampControlStructureDiagram_setFont_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/font.png",
        tooltip = "%StampControlStructureDiagram_setFont_tooltip",
        type = ItemKind.CHECK)
@MenuCommand(id = "stpa.SetFontCommand",
        category = "edit.font")
public class SetFontHandler {

    /**
     * 図要素の文字のフォントを設定するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final GolfAction action = new GolfAction("SetFont");
        action.actionPerformed(e);
    }

    /**
     * 図要素の文字のフォントを設定するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // ノートか、テキストがある場合、true
        return Selections.stream(selection).anyMatch(object -> isTarget(object));
    }

    private boolean isTarget(final Object object) {
        return isNote(object) || isText(object) || isComponent(object) || isSignal(object);
    }

    private boolean isSignal(Object object) {
        return object instanceof STAMPShape
                && DiagramElements.isModelElementPresent((STAMPShape) object, Signal.class);
    }

    private boolean isComponent(Object object) {
        return object instanceof STAMPShape
                && DiagramElements.isModelElementPresent((STAMPShape) object, Component.class);
    }

    private boolean isNote(final Object object) {
        return object instanceof STAMPShape
                && DiagramElements.isModelElementPresent((STAMPShape) object, Note.class);
    }

    private boolean isText(final Object object) {
        if (object instanceof Rectangle) {
            final Group group = ((Rectangle) object).getGroup();
            if (group != null) {
                return group.getMembers().stream().anyMatch(Text.class::isInstance);
            }
        }
        return false;
    }

}
