package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.dg.Image;
import net.astah.golf.control.GolfAction;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.stpa.stamp.notation.STAMPDiagram;
import net.astah.stpa.stamp.notation.STAMPGraphicalShape;
import net.astah.stpa.stamp.notation.STAMPLabel;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 図要素に線色を設定します。
 */
@Handler
@MenuItem(label = "%StampControlStructureDiagram_setLineColor_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/setcolorLine.png",
        tooltip = "%StampControlStructureDiagram_setLineColor_tooltip",
        type = ItemKind.CHECK)
@MenuCommand(id = "stpa.SetLineColorCommand",
        category = "edit.color")
public class SetLineColorHandler {

    /**
     * 図要素に線色を設定するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final GolfAction action = new GolfAction("SetLineColor");
        action.actionPerformed(e);
    }

    /**
     * 図要素に線色を設定するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 図、画像、STAMPLabel以外の要素があれば、true
        return Selections.stream(selection).anyMatch(object -> isTarget(object));
    }

    private boolean isTarget(final Object object) {
        return !(object instanceof STAMPDiagram
                || object instanceof STAMPLabel
                || isImage(object));
    }

    private boolean isImage(final Object object) {
        return object instanceof STAMPGraphicalShape
                && ((STAMPGraphicalShape) object).getFigure() instanceof Image;
    }

}
