package net.astah.stpa.stamp.ui.table;

import static net.astah.stpa.stamp.STAMPPackage.Literals.CONTROL_ACTION;
import static net.astah.stpa.stamp.STAMPPackage.Literals.CONTROL_LINK;
import static net.astah.stpa.stamp.STAMPPackage.Literals.CONTROL_LINK__ACTION;
import static net.astah.stpa.stamp.STAMPPackage.Literals.FEEDBACK;
import static net.astah.stpa.stamp.STAMPPackage.Literals.FEEDBACK_LINK;
import static net.astah.stpa.stamp.STAMPPackage.Literals.FEEDBACK_LINK__FEEDBACK;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;

import org.eclipse.emf.edit.provider.IItemPropertySource;

import net.astah.emf.edit.provider.swing.AdapterFactoryStyledLabelProvider;
import net.astah.golf.view.swing.IPropertySourceProvider;
import net.astah.golf.view.swing.MultiValueTableCellRenderer;
import net.astah.golf.widget.swing.IInputProvider;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.stpa.stamp.ui.view.swing.InputOutputDialogPanel;
import net.astah.stpa.stamp.ui.view.swing.SignalDialogPanel;
import net.astah.table.ui.swing.DialogCellEditor;

/**
 * コンポーネント抽出表の各項目を表示するための{@link TableCellRenderer} と{@link TableCellEditor}
 * を提供するクラスです。
 */
public class ComponentExtractingTableColumnProvider extends STPAGenericTableColumnProvider {
    private static final int TARGET_COLUMN_INDEX = 0;
    private static final int CONTROL_ACTION_COLUMN_INDEX = 3;
    private static final int FEEDBACK_COLUMN_INDEX = 4;
    private static final int INPUT_OUTPUT_COLUMN_INDEX = 5;
    private static final String IN_OUT_DIALOG_TITLE = "I/O";

    @Override
    protected void updateColumnModel(final TableColumnModel columnModel,
            final IInputProvider inputProvider,
            final IPropertySourceProvider propertySourceProvider) {
        super.updateColumnModel(columnModel, inputProvider, propertySourceProvider);
        final TableCellEditor cellEditor = columnModel.getColumn(0).getCellEditor();
        if (cellEditor instanceof ColumnCellEditor) {
            ((ColumnCellEditor) cellEditor).setClickCountToStart(0);
        }
    }

    @Override
    protected TableCellRenderer createCellRenderer(
            final TuplePropertyColumnDescriptor columnDescriptor,
            final IPropertySourceProvider propertySourceProvider) {
        final int index = columnDescriptor.getTuple().getIndex();
        if (index == TARGET_COLUMN_INDEX) {
            return new CheckBoxTableCellRenderer(columnDescriptor, propertySourceProvider);
        } else if (index == CONTROL_ACTION_COLUMN_INDEX
                || index == FEEDBACK_COLUMN_INDEX
                || index == INPUT_OUTPUT_COLUMN_INDEX) {
            return new MultiValueTableCellRenderer(columnDescriptor, propertySourceProvider,
                    new AdapterFactoryStyledLabelProvider(getAdapterFactory()));
        }
        return super.createCellRenderer(columnDescriptor, propertySourceProvider);
    }

    @Override
    protected TableCellEditor createCellEditor(TuplePropertyColumnDescriptor columnDescriptor,
            IPropertySourceProvider propertySourceProvider) {
        if (columnDescriptor.getTuple().getIndex() == CONTROL_ACTION_COLUMN_INDEX) {
            final DialogCellEditor dialogCellEditor = new DialogCellEditor(columnDescriptor,
                    propertySourceProvider,
                    new SignalDialogPanel(propertySourceProvider, CONTROL_LINK,
                            CONTROL_LINK__ACTION, CONTROL_ACTION) {
                        private static final long serialVersionUID = 1L;

                        @Override
                        protected String getSignalHeaderText() {
                            return Messages.signal_dialog_panel_control_action_header;
                        }
                    }, "CA");
            dialogCellEditor.setClickCountToStart(2);
            return dialogCellEditor;
        } else if (columnDescriptor.getTuple().getIndex() == FEEDBACK_COLUMN_INDEX) {
            final DialogCellEditor dialogCellEditor = new DialogCellEditor(columnDescriptor,
                    propertySourceProvider,
                    new SignalDialogPanel(propertySourceProvider, FEEDBACK_LINK,
                            FEEDBACK_LINK__FEEDBACK, FEEDBACK) {
                        private static final long serialVersionUID = 1L;

                        @Override
                        protected String getSignalHeaderText() {
                            return Messages.signal_dialog_panel_feedback_header;
                        }
                    }, "FB");
            dialogCellEditor.setClickCountToStart(2);
            return dialogCellEditor;
        } else if (columnDescriptor.getTuple().getIndex() == INPUT_OUTPUT_COLUMN_INDEX) {
            final DialogCellEditor dialogCellEditor = new DialogCellEditor(columnDescriptor,
                    propertySourceProvider,
                    new InputOutputDialogPanel(propertySourceProvider, CONTROL_ACTION),
                    IN_OUT_DIALOG_TITLE);
            dialogCellEditor.setClickCountToStart(2);
            return dialogCellEditor;
        } else {
            final ColumnCellEditor columnCellEditor = new ColumnCellEditor(columnDescriptor,
                    propertySourceProvider);
            columnCellEditor.setClickCountToStart(2);
            return columnCellEditor;
        }
    }

    /**
     * コンポーネント抽出表の対象列のチェックボックス用{@link DefaultTableCellRenderer} 拡張です。
     */
    class CheckBoxTableCellRenderer extends DefaultTableCellRenderer {
        private static final long serialVersionUID = -1445762715407663903L;

        public CheckBoxTableCellRenderer(PropertyColumnDescriptor propertyColumnDescriptor,
                IPropertySourceProvider propertySourceProvider) {
            super();
        }

        @Override
        public Component getTableCellRendererComponent(final JTable table, Object value,
                final boolean isSelected, final boolean hasFocus, final int row, final int column) {
            Component component = super.getTableCellRendererComponent(table, value, isSelected,
                    hasFocus, row, column);
            if (value instanceof IItemPropertySource) {
                Object editableValue = ((IItemPropertySource) value).getEditableValue(value);
                if (editableValue instanceof Boolean) {
                    JPanel panel = new JPanel();
                    panel.setLayout(new BorderLayout());
                    JCheckBox checkBox = new JCheckBox();
                    checkBox.setSelected((Boolean) editableValue);
                    checkBox.setForeground(super.getForeground());
                    checkBox.setBackground(super.getBackground());

                    panel.setFont(super.getFont());
                    panel.setForeground(super.getForeground());
                    panel.setBackground(super.getBackground());
                    panel.setBorder(super.getBorder());
                    panel.add(checkBox, BorderLayout.CENTER);
                    component = panel;
                }
            }
            return component;
        }
    }

}
