package net.astah.stpa.stamp.ui.view.swing;

import static java.util.Arrays.asList;
import static net.astah.stpa.stamp.STAMPPackage.Literals.COUNTERMEASURE__REMARKS;
import static net.astah.stpa.stamp.STAMPPackage.Literals.COUNTERMEASURE__TARGET_COMPONENT;
import static net.astah.stpa.stamp.STAMPPackage.Literals.HAZARD_CAUSAL_FACTOR;
import static net.astah.stpa.stamp.STAMPPackage.Literals.HAZARD_CAUSAL_FACTOR__COUNTERMEASURE;
import static net.astah.stpa.stamp.STAMPPackage.Literals.HAZARD_CAUSAL_FACTOR__UNSAFE_CONTROL_ACTION;
import static net.astah.stpa.stamp.STAMPPackage.Literals.IDENTIFIED_ELEMENT__DESCRIPTION;
import static net.astah.stpa.stamp.STAMPPackage.Literals.IDENTIFIED_ELEMENT__ID;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.dd.di.Diagram;
import net.astah.emf.common.ecore.EComparators;
import net.astah.emf.common.notify.EFeaturePath;
import net.astah.emf.edit.provider.swing.JTableBuilder;
import net.astah.emf.edit.provider.swing.PropertyEditorFactory;
import net.astah.emf.edit.provider.swing.PropertyEditorFactory.Registry;
import net.astah.emf.edit.provider.swing.PropertyEditorFactoryRegistryImpl;
import net.astah.emf.edit.provider.swing.editor.ManyEReferenceCellEditor;
import net.astah.golf.model.EntityStoreEvent;
import net.astah.golf.widget.CJComponent;
import net.astah.golf.widget.swing.GenericTable;
import net.astah.notation.TableView;
import net.astah.stpa.stamp.STPAAnalysis;
import net.astah.stpa.stamp.notation.CountermeasureTable;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.stpa.stamp.ui.menu.handler.AddCountermeasureHandler;
import net.astah.stpa.stamp.ui.menu.handler.DeleteCountermeasureHandler;
import net.astah.stpa.stamp.ui.menu.handler.SelectCountermeasureHandler;
import net.astah.stpa.stamp.ui.table.CountermeasureTableColumnProvider;
import net.astah.table.DynamicDataset;
import net.astah.table.ElementSet;
import net.astah.table.FixedDataset;
import net.astah.table.Grid;
import net.astah.table.JoinReference;
import net.astah.table.JoinType;
import net.astah.table.ObjectDataQuery;
import net.astah.table.ObjectElement;
import net.astah.table.PropertyCellDescriptor;
import net.astah.table.Table;
import net.astah.table.TableFactory;
import net.astah.table.ui.swing.GenericTableColumnProvider;
import net.astah.table.ui.swing.GenericTableTableTransferHandler;
import net.astah.ui.menu.annotation.Context;

/**
 * 対策表のTableEditorPeer 実装です。
 */
@Context(id = "net.astah.stpa.stamp.notation.CountermeasureTable",
        classes = { // classesの記述順でメニューを表示する
                AddCountermeasureHandler.class,
                SelectCountermeasureHandler.class,
                DeleteCountermeasureHandler.class
        })
public class CountermeasureTableEditorPeer extends STPATableEditorPeer {
    public static final int HAZARD_CAUSAL_FACTOR_INDEX = 0;
    public static final int COUNTERMEASURE_INDEX = 1;
    public static final int COUNTERMEASURE_ID_COLUMN = 2;
    public static final int COUNTERMEASURE_COLUMN = 3;

    private JPanel warnPanel;

    @Override
    public void initialize(final Diagram diagram, final CJComponent parent) {
        super.initialize(diagram, parent);
        final JTable table = (JTable) getTable().getComponent();
        table.setCellSelectionEnabled(true);
    }

    @Override
    protected float[] getColumnsWidthRatio() {
        return new float[] { 1 / 12f, 2 / 12f, 1 / 12f, 2 / 12f, 2 / 12f, 2 / 12f, 2 / 12f };
    }

    @Override
    public CountermeasureTable getDiagram() {
        return (CountermeasureTable) super.getDiagram();
    }

    @Override
    protected Container createNorthPanel() {
        final Container panel = super.createNorthPanel();
        panel.add(createWarnPanel(), 0);
        updateWarnPanel();
        return panel;
    }

    @Override
    public void update(EntityStoreEvent e) {
        super.update(e);
        updateWarnPanel();
    }

    private Component createWarnPanel() {
        warnPanel = new JPanel();
        warnPanel.setLayout(new BoxLayout(warnPanel, BoxLayout.Y_AXIS));
        warnPanel.setBorder(
                new CompoundBorder(warnPanel.getBorder(), new EmptyBorder(10, 10, 0, 0)));

        final JTextArea warnLabel = new JTextArea();
        warnLabel.setText(Messages.countermeasure_table_hcf_none_warning);
        warnLabel.setForeground(Color.RED);
        warnLabel.setOpaque(false);
        warnLabel.setEditable(false);
        warnLabel.setFocusable(false);
        warnLabel.setLineWrap(true);
        warnLabel.setFont(null);

        warnPanel.add(warnLabel);
        warnPanel.setVisible(false);
        return warnPanel;
    }

    private void updateWarnPanel() {
        final STPAAnalysis analysis = getScope();
        if (analysis != null) {
            warnPanel.setVisible(analysis.getHazardCausalFactors().isEmpty());
        }
    }

    @Override
    protected Table prepareTableModel(final TableView view) {
        final STPAAnalysis analysis = getScope();

        final ObjectDataQuery row0 = TableFactory.eINSTANCE.createObjectDataQuery();
        row0.getObjects().add(analysis);
        row0.setMetaclass(HAZARD_CAUSAL_FACTOR);
        row0.setTraversal(true);

        final JoinReference row1 = TableFactory.eINSTANCE.createJoinReference();
        applyFeaturePath(row1, EFeaturePath.build(HAZARD_CAUSAL_FACTOR__COUNTERMEASURE));
        row1.setJoinType(JoinType.OUTER);
        row1.setOrigin(row0);

        final DynamicDataset row = TableFactory.eINSTANCE.createDynamicDataset();
        row.getQueries().addAll(asList(row0, row1));

        final ObjectElement column0 = TableFactory.eINSTANCE.createObjectElement();
        column0.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.countermeasure_table_hcf_id_label));
        final PropertyCellDescriptor column0Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column0Cell.setOrigin(row.getQueries().indexOf(row0));
        applyFeaturePath(column0Cell, EFeaturePath.build(IDENTIFIED_ELEMENT__ID));
        column0.setCell(column0Cell);

        final ObjectElement column1 = TableFactory.eINSTANCE.createObjectElement();
        column1.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.countermeasure_table_hcf_label));
        final PropertyCellDescriptor column1Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column1Cell.setOrigin(row.getQueries().indexOf(row0));
        applyFeaturePath(column1Cell, EFeaturePath.build(IDENTIFIED_ELEMENT__DESCRIPTION));
        column1Cell.setLineWrap(true);
        column1.setCell(column1Cell);

        final ObjectElement column2 = TableFactory.eINSTANCE.createObjectElement();
        column2.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.countermeasure_table_countermeasure_id_label));
        final PropertyCellDescriptor column2Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column2Cell.setOrigin(row.getQueries().indexOf(row1));
        applyFeaturePath(column2Cell, EFeaturePath.build(IDENTIFIED_ELEMENT__ID));
        column2.setCell(column2Cell);

        final ObjectElement column3 = TableFactory.eINSTANCE.createObjectElement();
        column3.setHeader(TableFactory.eINSTANCE.createFixedCellDescriptor(
                Messages.countermeasure_table_countermeasure_description_label));
        final PropertyCellDescriptor column3Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column3Cell.setOrigin(row.getQueries().indexOf(row1));
        applyFeaturePath(column3Cell, EFeaturePath.build(IDENTIFIED_ELEMENT__DESCRIPTION));
        column3Cell.setLineWrap(true);
        column3.setCell(column3Cell);

        final ObjectElement column4 = TableFactory.eINSTANCE.createObjectElement();
        column4.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.countermeasure_table_uca_label));
        final PropertyCellDescriptor column4Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column4Cell.setOrigin(row.getQueries().indexOf(row0));
        column4Cell.setReadOnly(true);
        column4Cell.setLineWrap(true);
        applyFeaturePath(column4Cell,
                EFeaturePath.build(HAZARD_CAUSAL_FACTOR__UNSAFE_CONTROL_ACTION));
        column4.setCell(column4Cell);

        final ObjectElement column5 = TableFactory.eINSTANCE.createObjectElement();
        column5.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.countermeasure_table_components_label));
        final PropertyCellDescriptor column5Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column5Cell.setOrigin(row.getQueries().indexOf(row1));
        applyFeaturePath(column5Cell, EFeaturePath.build(COUNTERMEASURE__TARGET_COMPONENT));
        column5Cell.setLineWrap(true);
        column5.setCell(column5Cell);

        final ObjectElement column6 = TableFactory.eINSTANCE.createObjectElement();
        column6.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.countermeasure_table_remark_label));
        final PropertyCellDescriptor column6Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column6Cell.setOrigin(row.getQueries().indexOf(row1));
        applyFeaturePath(column6Cell, EFeaturePath.build(COUNTERMEASURE__REMARKS));
        column6Cell.setLineWrap(true);
        column6.setCell(column6Cell);

        final ElementSet columnSet = TableFactory.eINSTANCE.createElementSet();
        columnSet.getElements()
                .addAll(asList(column0, column1, column2, column3, column4, column5, column6));
        final FixedDataset column = TableFactory.eINSTANCE.createFixedDataset();
        column.getElementSets().add(columnSet);

        final Table table = TableFactory.eINSTANCE.createTable();
        table.getColumns().add(column);
        table.getRows().add(row);

        return table;
    }

    /**
     * 対策表のスコープを取得します。
     * 
     * @return 対策表のスコープ
     */
    public STPAAnalysis getScope() {
        return getDiagram().getScope();
    }

    @Override
    protected JTable createJTable(final AdapterFactory adapterFactory, final Grid grid) {
        final GenericTableColumnProvider columnProvider = new CountermeasureTableColumnProvider();
        final PropertyEditorFactory.Registry registry = new PropertyEditorFactoryRegistryImpl(
                Registry.INSTANCE);

        // コンポーネントを対象としたセルエディタ（のファクトリ）を作成
        final PropertyEditorFactory factory = new ManyEReferenceCellEditor.DescriptorImpl(
                getScope(),
                EComparators.nameIgnoreCase().thenComparing(EComparators.eObjectPath()));

        registry.getFeatureMap().put(COUNTERMEASURE__TARGET_COMPONENT, factory);

        columnProvider.setAdapterFactory(adapterFactory);
        columnProvider.setGrid(grid);
        final TransactionalEditingDomain txDomain = TransactionUtil.getEditingDomain(getDiagram());
        return JTableBuilder.builder(txDomain)
                .adapterFactory(adapterFactory)
                .columnProvider(columnProvider)
                .transferHandler(new GenericTableTableTransferHandler(txDomain, domain)).input(grid)
                .propertyEditorFactoryRegistry(registry)
                .font(resolveTableFont())
                .reorderingAllowed(false)
                .build(new GenericTable());
    }

    @Override
    protected boolean needToShowMessageOnActivated() {
        // HCFが無い場合はなにもできないので、操作ガイドは表示しないでおく
        if (getDiagram().getScope().getHazardCausalFactors().isEmpty()) {
            return false;
        }

        return getDiagram().getScope().getCountermeasures().isEmpty();
    }
}
