package net.astah.stpa.stamp.ui.view.swing;

import static java.util.Collections.singleton;
import static net.astah.stpa.stamp.STAMPPackage.Literals.IDENTIFIED_ELEMENT__ID;
import static net.astah.stpa.stamp.STAMPPackage.Literals.UNSAFE_CONTROL_ACTION__VIOLATING_SAFETY_CONSTRAINT;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import java.util.stream.Collectors;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.command.IdentityCommand;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EObjectEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.command.CreateChildCommand;
import org.eclipse.emf.edit.command.MoveCommand;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor.PropertyValueWrapper;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.uml2.common.util.UML2Util;

import com.google.common.collect.Iterables;
import com.google.common.collect.Sets;

import net.astah.emf.common.ecore.EComparators;
import net.astah.emf.common.notify.EFeaturePath;
import net.astah.emf.edit.command.DeferringCommandWrapper;
import net.astah.emf.edit.command.DestroyCommand;
import net.astah.emf.edit.job.CommandJob;
import net.astah.emf.edit.job.Jobs;
import net.astah.emf.edit.provider.ItemProviders;
import net.astah.emf.edit.provider.swing.AdapterFactoryContentProvider;
import net.astah.emf.edit.provider.swing.FeaturePathContentProvider;
import net.astah.emf.edit.provider.swing.JTableBuilder;
import net.astah.emf.edit.provider.swing.PropertyEditorFactory;
import net.astah.emf.edit.provider.swing.PropertyEditorFactory.Registry;
import net.astah.emf.edit.provider.swing.PropertyEditorFactoryRegistryImpl;
import net.astah.emf.edit.provider.swing.TransactionalFeaturePathContentProvider;
import net.astah.emf.edit.provider.swing.editor.ManyEReferenceCellEditor;
import net.astah.golf.view.swing.IPropertySourceProvider;
import net.astah.golf.view.swing.TableColumnProvider;
import net.astah.golf.view.swing.TableColumnProvider.IdPropertyColumnDescriptor;
import net.astah.golf.widget.swing.ContentTableModel;
import net.astah.golf.widget.swing.GenericTable;
import net.astah.golf.widget.swing.IViewerAdapter;
import net.astah.golf.widget.swing.SwingIconRegistry;
import net.astah.jomt.jview.EditableTextComponentMouseAdapther;
import net.astah.stpa.stamp.ControlAction;
import net.astah.stpa.stamp.Element;
import net.astah.stpa.stamp.Note;
import net.astah.stpa.stamp.STAMPFactory;
import net.astah.stpa.stamp.STAMPPackage;
import net.astah.stpa.stamp.STPAAnalysis;
import net.astah.stpa.stamp.SafetyConstraint;
import net.astah.stpa.stamp.UCAAnalysis;
import net.astah.stpa.stamp.UCAAnalysisResult;
import net.astah.stpa.stamp.UCAGuideWord;
import net.astah.stpa.stamp.UnsafeControlAction;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.table.Tuple;
import net.astah.table.ui.Tables;
import net.astah.table.ui.swing.TableCellDialogPanel;

/**
 * UCA表にUCA, ノート を追加するダイアログのパネル実装です。
 */
public class UCADialogPanel extends TableCellDialogPanel implements ActionListener {
    private static final long serialVersionUID = -7596620116731878186L;
    private static final int DIALOG_WIDTH = 650;
    private static final int DIALOG_HEIGHT = 200;

    private JTable ucaTable;
    private JButton addUCAButton;
    private JButton addNonUCAButton;
    private JButton deleteButton;
    private JButton upButton;
    private JButton downButton;

    private UCAAnalysis ucaAnalysis;
    private UCAAnalysisResult ucaAnalysisResult;
    private UCAGuideWord guideWord;
    private ControlAction controlAction;

    public UCADialogPanel(final IPropertySourceProvider propertySourceProvider) {
        super(propertySourceProvider);
        setPreferredSize(new Dimension(SwingIconRegistry.scaleValue(DIALOG_WIDTH),
                SwingIconRegistry.scaleValue(DIALOG_HEIGHT)));
    }

    /**
     * カラムの幅比率を返します。
     * 
     * @return 幅比率群
     */
    protected float[] getColumnsWidthRatio() {
        return new float[] { 0.08f, 0.15f, 0.49f, 0.28f };
    }

    @Override
    protected void setInput(final Object object, final JTable baseTable, final int row,
            final int column) {
        final ContentTableModel baseTableModel = (ContentTableModel) baseTable.getModel();
        final Tuple tempTuple = (Tuple) baseTableModel.getRowAt(row);
        final Tuple ucaTableRowTuple = tempTuple.getGrid().getRowTuples().get(row);
        final Tuple ucaTableColumnTuple = tempTuple.getGrid().getColumnTuples().get(column);

        // tempTuple
        ucaAnalysis = (UCAAnalysis) ucaTableRowTuple
                .getElementObject(UCATableEditorPeer.UCA_ANALYSIS_TUPLE_ELEMENT_INDEX);
        guideWord = (UCAGuideWord) ucaTableColumnTuple
                .getElementObject(UCATableEditorPeer.GUIDEWORD_TUPLE_ELEMENT_INDEX);
        controlAction = (ControlAction) ucaTableRowTuple
                .getElementObject(UCATableEditorPeer.CONTROL_ACTION_TUPLE_ELEMENT_INDEX);

        if (object instanceof EObjectEList<?>) {
            EObject parent = ((EObjectEList<?>) object).getEObject();
            if (parent instanceof UCAAnalysisResult) {
                ucaAnalysisResult = (UCAAnalysisResult) parent;
            }
        }

        final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(ucaAnalysis);
        final AdapterFactory adapterFactory = ItemProviders.getAdapterFactory(domain);
        final AdapterFactoryContentProvider contentProvider = createContentProvider(domain,
                adapterFactory);

        final PropertyEditorFactory.Registry registry = new PropertyEditorFactoryRegistryImpl(
                Registry.INSTANCE);

        // 安全制約を対象としたセルエディタ（のファクトリ）を作成
        final PropertyEditorFactory factory = new ManyEReferenceCellEditor.DescriptorImpl(
                ucaAnalysis.getOwner(),
                EComparators.comparingFeature(IDENTIFIED_ELEMENT__ID.getName())
                        .thenComparing(EComparators.eObjectPath()));

        registry.getFeatureMap().put(UNSAFE_CONTROL_ACTION__VIOLATING_SAFETY_CONSTRAINT, factory);
        contentProvider.setPropertyEditorFactoryRegistry(registry);

        JTableBuilder.builder(domain).adapterFactory(adapterFactory)
                .addColumn(
                        new CheckBoxColumnDescriptor(Messages.uca_dialog_panel_is_uca_check_header))
                .addColumn(TableColumnProvider
                        .idProperty(Messages.uca_dialog_panel_uca_id_header, "id")
                        .setClickCountToStart(2))
                .addColumn(TableColumnProvider
                        .idProperty(Messages.uca_dialog_panel_text_header,
                                Sets.newHashSet("description", "body"))
                        .setLineWrap(true)
                        .setClickCountToStart(2))
                .addColumn(new SafetyConstraintColumnDescriptor("violatingSafetyConstraint")
                        .setHeaderValue(Messages.uca_dialog_panel_violating_sc_header)
                        .setLineWrap(true)
                        .setClickCountToStart(2))
                .contentProvider(contentProvider)
                .propertyEditorFactoryRegistry(registry)
                .reorderingAllowed(false)
                .input(ucaAnalysisResult)
                .build(ucaTable);

        ucaTable.setFont(baseTable.getFont());
        ucaTable.getTableHeader().setFont(baseTable.getFont());

        // FIXME この時点ではTableのWidthが確定していないため、ダイアログのサイズを元に設定する。
        // 4列あるが3列目までを設定し、4列目はパディング分の誤差を辻褄合わせするため設定しない
        final Enumeration<TableColumn> columns = ucaTable.getColumnModel().getColumns();
        for (final float ratio : getColumnsWidthRatio()) {
            if (columns.hasMoreElements()) {
                columns.nextElement().setPreferredWidth((int) (DIALOG_WIDTH * ratio));
            }
        }
    }

    @Override
    protected void initComponent(final IPropertySourceProvider propertySourceProvider) {
        ucaTable = new GenericTable();
        EditableTextComponentMouseAdapther.install(ucaTable);
        final JScrollPane scrollPane = new JScrollPane(ucaTable);
        addUCAButton = new JButton(Messages.uca_dialog_panel_uca_add_button);
        addUCAButton.addActionListener(this);
        addNonUCAButton = new JButton(Messages.uca_dialog_panel_non_uca_add_button);
        addNonUCAButton.addActionListener(this);
        deleteButton = new JButton(Messages.stpa_dialog_panel_delete_button);
        deleteButton.addActionListener(this);
        deleteButton.setEnabled(false);
        upButton = new JButton(Messages.stpa_dialog_panel_up_button);
        upButton.addActionListener(this);
        upButton.setEnabled(false);
        downButton = new JButton(Messages.stpa_dialog_panel_down_button);
        downButton.addActionListener(this);
        downButton.setEnabled(false);

        ucaTable.getSelectionModel().addListSelectionListener(listSelectionEvent -> {
            if (!listSelectionEvent.getValueIsAdjusting()) {
                int selectedCount = ucaTable.getSelectedRowCount();
                deleteButton.setEnabled(selectedCount > 0);
                upButton.setEnabled(selectedCount == 1);
                downButton.setEnabled(selectedCount == 1);
            }
        });

        final GridBagLayout layout = new GridBagLayout();
        setLayout(layout);
        final GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets = new Insets(2, 10, 2, 10);
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 1;
        add(scrollPane, gbc);

        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.insets = new Insets(2, 10, 2, 10);
        gbc.weightx = 0.5;
        gbc.weighty = 0;
        add(new JLabel(), gbc);

        gbc.gridx = 0;
        gbc.gridy = 1;
        add(addUCAButton, gbc);
        gbc.gridx = 1;
        gbc.gridy = 1;
        add(addNonUCAButton, gbc);
        gbc.gridx = 2;
        gbc.gridy = 1;
        add(deleteButton, gbc);
        gbc.gridx = 3;
        gbc.gridy = 1;
        add(upButton, gbc);
        gbc.gridx = 4;
        gbc.gridy = 1;
        add(downButton, gbc);
    }

    /**
     * テーブルに表示するオブジェクトを供給するプロバイダを構築します。
     * 
     * @param domain
     *            {@code TransactionalEditingDomain}
     * @param adapterFactory
     *            {@code AdapterFactory}
     * @return 構築したプロバイダ
     */
    protected AdapterFactoryContentProvider createContentProvider(
            final TransactionalEditingDomain domain,
            final AdapterFactory adapterFactory) {
        final FeaturePathContentProvider contentProvider = new TransactionalFeaturePathContentProvider(
                domain, adapterFactory);
        contentProvider.addPath(
                EFeaturePath.build(STAMPPackage.Literals.UCA_ANALYSIS_RESULT__RESULT_ELEMENT));
        return contentProvider;
    }

    @Override
    public void actionPerformed(final ActionEvent e) {
        final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(ucaAnalysis);
        final int selectedRow = ucaTable.getSelectedRow();

        final Command command;
        final int newSelectionRow;
        if (e.getSource() == addUCAButton) {
            command = createAddUCACommands(domain, selectedRow);
            if (selectedRow == -1) {
                newSelectionRow = ucaTable.getRowCount();
            } else {
                newSelectionRow = selectedRow + 1;
            }
        } else if (e.getSource() == addNonUCAButton) {
            command = createAddNonUCACommands(domain, selectedRow);
            if (selectedRow == -1) {
                newSelectionRow = ucaTable.getRowCount();
            } else {
                newSelectionRow = selectedRow + 1;
            }

        } else if (e.getSource() == deleteButton) {
            command = createDeleteCommands(domain, ucaTable.getSelectedRows());
            newSelectionRow = -1;
        } else if (e.getSource() == upButton) {
            command = createMoveUpCommand(domain, selectedRow);
            newSelectionRow = selectedRow - 1;
        } else if (e.getSource() == downButton) {
            command = createMoveDownCommand(domain, selectedRow);
            newSelectionRow = selectedRow + 1;
        } else {
            return;
        }
        // 対象の分析結果が変わっている場合はテーブルを更新する
        final IViewerAdapter model = (IViewerAdapter) ucaTable.getModel();
        if (model.getInput() != ucaAnalysisResult) {
            model.setInput(ucaAnalysisResult);
            model.refresh();
        }

        final CommandJob job = new CommandJob(domain, command);
        Jobs.toCompletableFuture(job, true).thenRun(() -> {
            if (newSelectionRow >= 0) {
                ucaTable.scrollRectToVisible(ucaTable.getCellRect(newSelectionRow, 0, true));
                ucaTable.setRowSelectionInterval(newSelectionRow, newSelectionRow);
            }
            if (e.getSource() == addUCAButton || e.getSource() == addNonUCAButton) {
                Tables.startCellEditing(ucaTable, newSelectionRow, 2);
            }
        });
        job.schedule();
    }

    @Override
    protected void okHandler() {
        final TableCellEditor cellEditor = ucaTable.getCellEditor();
        if (cellEditor != null) {
            cellEditor.stopCellEditing();
        }
        closeDialog();
    }

    @Override
    protected void cancelHandler() {
        final TableCellEditor cellEditor = ucaTable.getCellEditor();
        if (cellEditor != null) {
            cellEditor.cancelCellEditing();
        }
        closeDialog();
    }

    /**
     * UCA追加コマンドを作成します。
     * 
     * @param domain
     *            ドメイン
     * @param selectedRow
     *            選択行
     * @return UCA追加コマンド
     */
    protected Command createAddUCACommands(final TransactionalEditingDomain domain,
            final int selectedRow) {
        final UnsafeControlAction uca = STAMPFactory.eINSTANCE.createUnsafeControlAction();
        uca.setDescription("UCA");
        uca.setControlAction(controlAction);
        final CompoundCommand command = new CompoundCommand();
        final STPAAnalysis stpaAnalysis = ucaAnalysis.getOwner();
        if (ucaAnalysisResult == null) {
            command.append(createAddUCAAnalysisCommand(domain));
        }

        command.append(CreateChildCommand.create(domain, stpaAnalysis,
                new CommandParameter(stpaAnalysis,
                        STAMPPackage.Literals.STPA_ANALYSIS__UNSAFE_CONTROL_ACTION, uca),
                Collections.singleton(stpaAnalysis))); // IDの採番のため追加して移動する
        command.append(AddCommand.create(domain, ucaAnalysisResult,
                STAMPPackage.Literals.UCA_ANALYSIS_RESULT__UNSAFE_CONTROL_ACTION, uca));
        command.append(new DeferringCommandWrapper(MoveCommand.create(domain, ucaAnalysisResult,
                STAMPPackage.Literals.UCA_ANALYSIS_RESULT__RESULT_ELEMENT, uca,
                (selectedRow < 0) ? CommandParameter.NO_INDEX : (selectedRow + 1))));

        return command.unwrap();
    }

    /**
     * 非UCA追加コマンドを作成します。
     * 
     * @param domain
     *            ドメイン
     * @param selectedRow
     *            選択行
     * @return 非UCA追加コマンド
     */
    protected Command createAddNonUCACommands(final TransactionalEditingDomain domain,
            final int selectedRow) {
        CompoundCommand command = new CompoundCommand();
        final Note note = STAMPFactory.eINSTANCE.createNote();
        note.setBody("-");
        if (ucaAnalysisResult == null) {
            command.append(createAddUCAAnalysisCommand(domain));
        }

        command.append(CreateChildCommand.create(
                domain, ucaAnalysisResult, new CommandParameter(ucaAnalysisResult,
                        STAMPPackage.Literals.ELEMENT__OWNED_NOTE, note),
                singleton(ucaAnalysisResult)));
        command.append(new DeferringCommandWrapper(MoveCommand.create(domain, ucaAnalysisResult,
                STAMPPackage.Literals.UCA_ANALYSIS_RESULT__RESULT_ELEMENT, note,
                (selectedRow < 0) ? CommandParameter.NO_INDEX : (selectedRow + 1))));
        return command.unwrap();
    }

    private Command createAddUCAAnalysisCommand(final TransactionalEditingDomain domain) {
        ucaAnalysisResult = STAMPFactory.eINSTANCE.createUCAAnalysisResult();
        ucaAnalysisResult.setControlAction(controlAction);
        ucaAnalysisResult.setGuideWord(guideWord);

        return AddCommand.create(domain, ucaAnalysis, STAMPPackage.Literals.UCA_ANALYSIS__RESULT,
                ucaAnalysisResult);
    }

    /**
     * UCA又は非UCAの削除コマンドを作成します。
     * 
     * @param domain
     *            ドメイン
     * @param rowIndices
     *            選択行
     * @return UCA又は非UCAの削除コマンド
     */
    protected Command createDeleteCommands(final TransactionalEditingDomain domain,
            final int[] rowIndices) {
        if ((rowIndices != null) && (rowIndices.length > 0)) {
            // 参照している要素と所有している要素に分類する
            final EList<Element> resultElements = ucaAnalysisResult.getResultElements();
            final List<Element> ownedElements = new ArrayList<>();
            final List<Element> elements = new ArrayList<>();
            for (final int rowIndex : rowIndices) {
                final Element element = resultElements.get(rowIndex);
                if (EcoreUtil.isAncestor(ucaAnalysisResult, element)) {
                    ownedElements.add(element);
                } else {
                    elements.add(element);
                }
            }

            final CompoundCommand result = new CompoundCommand();
            if (!ownedElements.isEmpty()) {
                // 所有している要素は削除する
                result.append(DestroyCommand.create(domain, ownedElements));
            }
            if (!elements.isEmpty()) {
                // 参照している要素は参照を解除することで必要に応じて削除する
                result.append(DestroyCommand.create(domain,
                        new DestroyCommand.Reference(ucaAnalysisResult,
                                STAMPPackage.Literals.UCA_ANALYSIS_RESULT__RESULT_ELEMENT,
                                elements)));
            }
            return result.unwrap();
        }
        return IdentityCommand.INSTANCE;
    }

    /**
     * 行の入れ替え（上方向）コマンドを作成します。
     * 
     * @param domain
     *            ドメイン
     * @param rowIndex
     *            選択行
     * @return 行の入れ替え（上方向）コマンド
     */
    protected Command createMoveUpCommand(TransactionalEditingDomain domain, int rowIndex) {
        if (rowIndex < 1) {
            return IdentityCommand.INSTANCE;
        }
        final Element element = ucaAnalysisResult.getResultElements().get(rowIndex);
        return MoveCommand.create(domain, ucaAnalysisResult,
                STAMPPackage.Literals.UCA_ANALYSIS_RESULT__RESULT_ELEMENT, element, rowIndex - 1);
    }

    /**
     * 行の入れ替え（下方向）コマンドを作成します。
     * 
     * @param domain
     *            ドメイン
     * @param rowIndex
     *            選択行
     * @return 行の入れ替え（下方向）コマンド
     */
    protected Command createMoveDownCommand(TransactionalEditingDomain domain, int rowIndex) {
        if (rowIndex >= ucaAnalysisResult.getResultElements().size() - 1) {
            return IdentityCommand.INSTANCE;
        }
        final Element element = ucaAnalysisResult.getResultElements().get(rowIndex);
        return MoveCommand.create(domain, ucaAnalysisResult,
                STAMPPackage.Literals.UCA_ANALYSIS_RESULT__RESULT_ELEMENT, element, rowIndex + 1);
    }

    private void closeDialog() {
        JTableBuilder.dispose(ucaTable);
        if (ucaAnalysisResult != null) {
            ucaAnalysisResult = null;
        }
    }

    /**
     * UCA行のチェックボックス用{@link TableColumnProvider.CheckBoxPropertyColumnDescriptor}
     * 実装です。
     */
    class CheckBoxColumnDescriptor extends TableColumnProvider.CheckBoxPropertyColumnDescriptor {

        public CheckBoxColumnDescriptor(final String header) {
            super(header);
            setHeaderValue(header);
        }

        @Override
        public Object getColumnValue(final IPropertySourceProvider propertySourceProvider,
                final Object object) {
            if (object == null) {
                return null;
            } else if (object instanceof UnsafeControlAction) {
                return true;
            } else {
                return false;
            }
        }

        @Override
        public void setColumnValue(final IPropertySourceProvider propertySourceProvider,
                final Object object, final Object value) {
            final CompoundCommand command = new CompoundCommand();
            final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(object);
            command.append(DestroyCommand.create(domain, singleton(object)));
            final EList<Element> supersetList = ucaAnalysisResult.getResultElements();
            final int index = supersetList.indexOf(object);

            if (object instanceof UnsafeControlAction) { // change UCA to Note
                final UnsafeControlAction uca = (UnsafeControlAction) object;
                final UCAAnalysisResult ucaAnalysisResult = Iterables
                        .tryFind(UML2Util.getInverseReferences(uca),
                                setting -> setting.getEObject() instanceof UCAAnalysisResult)
                        .transform(setting -> (UCAAnalysisResult) setting.getEObject()).orNull();
                final Note note = STAMPFactory.eINSTANCE.createNote();
                note.setBody(uca.getDescription());

                final int subsetListIndex = findNearlyIndex(supersetList,
                        ucaAnalysisResult.getOwnedNotes(), index);
                command.append(CreateChildCommand.create(domain, ucaAnalysisResult,
                        new CommandParameter(ucaAnalysisResult,
                                STAMPPackage.Literals.ELEMENT__OWNED_NOTE, note, subsetListIndex),
                        singleton(ucaAnalysisResult)));
                command.append(new DeferringCommandWrapper(MoveCommand.create(domain,
                        ucaAnalysisResult,
                        STAMPPackage.Literals.UCA_ANALYSIS_RESULT__RESULT_ELEMENT, note, index)));

            } else if (object instanceof Note) { // change Note to UCA
                final Note note = (Note) object;
                final UCAAnalysisResult ucaAnalysisResult = (UCAAnalysisResult) note.eContainer();
                final STPAAnalysis stpaAnalysis = (STPAAnalysis) ucaAnalysisResult.eContainer()
                        .eContainer();

                final UnsafeControlAction uca = STAMPFactory.eINSTANCE.createUnsafeControlAction();
                if (note.getBody().isEmpty()) {
                    uca.setDescription("UCA");
                } else {
                    uca.setDescription(note.getBody());
                }
                uca.setControlAction(ucaAnalysisResult.getControlAction());

                final int subsetListIndex = findNearlyIndex(supersetList,
                        ucaAnalysisResult.getUnsafeControlActions(), index);

                command.append(CreateChildCommand.create(domain, stpaAnalysis,
                        new CommandParameter(stpaAnalysis,
                                STAMPPackage.Literals.STPA_ANALYSIS__UNSAFE_CONTROL_ACTION, uca),
                        singleton(stpaAnalysis)));
                command.append(AddCommand.create(domain, ucaAnalysisResult,
                        STAMPPackage.Literals.UCA_ANALYSIS_RESULT__UNSAFE_CONTROL_ACTION, uca,
                        subsetListIndex));
                command.append(new DeferringCommandWrapper(MoveCommand.create(domain,
                        ucaAnalysisResult,
                        STAMPPackage.Literals.UCA_ANALYSIS_RESULT__RESULT_ELEMENT, uca, index)));
            }

            new CommandJob(domain, command.unwrap()).schedule();
        }

        private int findNearlyIndex(final List<? extends EObject> supersetList,
                final List<? extends EObject> subsetList, final int index) {
            final EObject nearlySametypeObj = Iterables
                    .tryFind(supersetList.subList(index, supersetList.size()),
                            subsetList::contains)
                    .orNull();
            if (nearlySametypeObj == null) {
                return subsetList.size();
            }
            return subsetList.indexOf(nearlySametypeObj);
        }
    }

    /**
     * 安全制約列用{@link IdPropertyColumnDescriptor} 実装です。
     */
    class SafetyConstraintColumnDescriptor extends IdPropertyColumnDescriptor {

        protected SafetyConstraintColumnDescriptor(Object propertyId) {
            super(propertyId);
        }

        @Override
        protected TableCellRenderer createCellRenderer(
                IPropertySourceProvider propertySourceProvider) {
            return new DefaultTableCellRenderer() {
                private static final long serialVersionUID = 2127237340620063003L;

                @Override
                public Component getTableCellRendererComponent(JTable table, Object value,
                        boolean isSelected, boolean hasFocus, int row, int column) {
                    Component component = super.getTableCellRendererComponent(table, value,
                            isSelected, hasFocus, row, column);
                    final JPanel panel = new JPanel();
                    panel.setFont(table.getFont());
                    if (isSelected) {
                        panel.setBackground(table.getSelectionBackground());
                    } else {
                        panel.setBackground(component.getBackground());
                    }

                    if (value instanceof PropertyValueWrapper && ((PropertyValueWrapper) value)
                            .getEditableValue(null) instanceof List) {
                        final List<?> list = (List<?>) ((PropertyValueWrapper) value)
                                .getEditableValue(null);
                        if (list.isEmpty()) {
                            return panel;
                        }
                        final FlowLayout flowLayout = new FlowLayout(FlowLayout.LEFT, 0, 0);
                        panel.setLayout(flowLayout);

                        int panelPreferredSize = 0;
                        String safetyConstraintString = list.stream()
                                .filter(element -> element instanceof SafetyConstraint)
                                .map(sc -> ((SafetyConstraint) sc).getId())
                                .collect(Collectors.joining(", "));

                        final JTextArea textArea = new JTextArea(safetyConstraintString);
                        if (isSelected) {
                            textArea.setForeground(Color.white);
                            textArea.setBackground(table.getSelectionBackground());
                        } else {
                            textArea.setForeground(Color.black);
                            textArea.setBackground(table.getBackground());
                        }

                        textArea.setFont(table.getFont());
                        textArea.setLineWrap(true);
                        textArea.doLayout();
                        textArea.setSize(table.getColumnModel().getColumn(column).getWidth(),
                                (int) textArea.getPreferredSize().getHeight());
                        panel.add(textArea);
                        panelPreferredSize += textArea.getPreferredSize().getHeight();

                        panel.doLayout();
                        panel.setPreferredSize(new Dimension(
                                table.getColumnModel().getColumn(column).getWidth(), (int) Math
                                        .max(getPreferredSize().getHeight(), panelPreferredSize)));
                        return panel;
                    }

                    return component;
                }
            };
        }
    }

}
