package net.astah.stpa.stamp.ui.view.swing;

import static java.util.Arrays.asList;
import static net.astah.stpa.stamp.STAMPPackage.Literals.CONTROL_ACTION;
import static net.astah.stpa.stamp.STAMPPackage.Literals.LINK__SOURCE;
import static net.astah.stpa.stamp.STAMPPackage.Literals.LINK__TARGET;
import static net.astah.stpa.stamp.STAMPPackage.Literals.NAMED_ELEMENT__NAME;
import static net.astah.stpa.stamp.STAMPPackage.Literals.SIGNAL__LINK;
import static net.astah.stpa.stamp.STAMPPackage.Literals.SIGNAL__PROVIDING_CONDITION;
import static net.astah.stpa.stamp.STAMPPackage.Literals.UCA_ANALYSIS__CONTROL_ACTION;
import static org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain.getEditingDomainFor;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.IdentityCommand;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.emf.edit.command.MoveCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.provider.IItemProviderDecorator;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.dd.di.Diagram;
import net.astah.emf.common.notify.EFeaturePath;
import net.astah.emf.edit.provider.ItemProviders;
import net.astah.emf.edit.provider.swing.JTableBuilder;
import net.astah.golf.model.EntityStoreEvent;
import net.astah.golf.widget.CJComponent;
import net.astah.golf.widget.IListSelection;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.ITableSelection;
import net.astah.golf.widget.TableSelection;
import net.astah.golf.widget.swing.GenericTable;
import net.astah.golf.widget.swing.Selections;
import net.astah.golf.widget.swing.SwingIconRegistry;
import net.astah.notation.TableView;
import net.astah.stpa.stamp.ControlAction;
import net.astah.stpa.stamp.STAMPPackage;
import net.astah.stpa.stamp.UCAAnalysis;
import net.astah.stpa.stamp.UCAGuideWordSet;
import net.astah.stpa.stamp.notation.UCATable;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.stpa.stamp.ui.menu.handler.JumpToDiagramElementHandler;
import net.astah.stpa.stamp.ui.table.UCAAnalysisResultCellEvaluator;
import net.astah.stpa.stamp.ui.table.UcaTableColumnProvider;
import net.astah.table.DataQuery;
import net.astah.table.DynamicDataset;
import net.astah.table.ElementSet;
import net.astah.table.FixedDataset;
import net.astah.table.Grid;
import net.astah.table.IQueryExecutor;
import net.astah.table.JoinReference;
import net.astah.table.JoinType;
import net.astah.table.NativeCellDescriptor;
import net.astah.table.NativeDataQuery;
import net.astah.table.ObjectDataQuery;
import net.astah.table.ObjectElement;
import net.astah.table.PropertyCellDescriptor;
import net.astah.table.PseudoCellDescriptor;
import net.astah.table.Table;
import net.astah.table.TableFactory;
import net.astah.table.Tuple;
import net.astah.table.edit.provider.GenericTableAdapterFactory;
import net.astah.table.edit.provider.GridItemProviderDecorator;
import net.astah.table.ui.swing.GenericTableColumnProvider;
import net.astah.table.ui.swing.GenericTableTableTransferHandler;
import net.astah.ui.menu.annotation.Context;

/**
 * UCA表のTableEditorPeer 実装です。
 *
 */
@Context(id = "net.astah.stpa.stamp.notation.UCATable",
        classes = { // classesの記述順でメニューを表示する
                JumpToDiagramElementHandler.class
        })
public class UCATableEditorPeer extends STPATableEditorPeer {

    public static final String ANNOTATION__UCA_TABLE = "ucaTable";
    public static final int NUMBER_COLUMN_WIDTH = 35;
    public static final int UCA_ANALYSIS_TUPLE_ELEMENT_INDEX = 3;
    public static final int GUIDEWORD_TUPLE_ELEMENT_INDEX = 0;
    public static final int CONTROL_ACTION_TUPLE_ELEMENT_INDEX = 0;

    private JPanel warnPanel;

    @Override
    public void initialize(final Diagram diagram, final CJComponent parent) {
        super.initialize(diagram, parent);

        final JTable table = (JTable) getTable().getComponent();
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setCellSelectionEnabled(true);

        table.getColumnModel().getColumn(0)
                .setMinWidth(SwingIconRegistry.scaleValue(NUMBER_COLUMN_WIDTH));
        table.getColumnModel().getColumn(0)
                .setMaxWidth(SwingIconRegistry.scaleValue(NUMBER_COLUMN_WIDTH));
    }

    @Override
    public UCATable getDiagram() {
        return (UCATable) super.getDiagram();
    }

    @Override
    protected Container createNorthPanel() {
        final Container panel = super.createNorthPanel();
        panel.add(createWarnPanel(), 0);
        updateWarnPanel();
        return panel;
    }

    @Override
    public void update(EntityStoreEvent e) {
        super.update(e);
        updateWarnPanel();
    }

    private Component createWarnPanel() {
        warnPanel = new JPanel();
        warnPanel.setLayout(new BoxLayout(warnPanel, BoxLayout.Y_AXIS));
        warnPanel.setBorder(
                new CompoundBorder(warnPanel.getBorder(), new EmptyBorder(10, 10, 0, 0)));

        final JTextArea warnLabel = new JTextArea();
        warnLabel.setText(Messages.uca_table_ca_none_warning);
        warnLabel.setForeground(Color.RED);
        warnLabel.setOpaque(false);
        warnLabel.setEditable(false);
        warnLabel.setFocusable(false);
        warnLabel.setLineWrap(true);
        warnLabel.setFont(null);

        warnPanel.add(warnLabel);
        warnPanel.setVisible(false);
        return warnPanel;
    }

    private void updateWarnPanel() {
        warnPanel.setVisible(getDiagram().getControlActions().isEmpty());
    }

    @Override
    protected Table prepareTableModel(final TableView view) {
        final UCATable ucaTable = (UCATable) view;
        // 対象のUCA表をDataQueryのannotationに入れる
        final EAnnotation annotation = EcoreFactory.eINSTANCE.createEAnnotation();
        annotation.setSource(ANNOTATION__UCA_TABLE);
        annotation.getReferences().add(ucaTable);

        final NativeDataQuery row0 = TableFactory.eINSTANCE.createNativeDataQuery();
        row0.getEAnnotations().add(annotation);
        row0.setImplementationClass(ControlActionFactory.class);
        row0.setMetaclass(CONTROL_ACTION);

        final JoinReference row1 = TableFactory.eINSTANCE.createJoinReference();
        applyFeaturePath(row1, EFeaturePath.build(SIGNAL__LINK, LINK__SOURCE));
        row1.setJoinType(JoinType.INNER);
        row1.setOrigin(row0);

        final JoinReference row2 = TableFactory.eINSTANCE.createJoinReference();
        applyFeaturePath(row2, EFeaturePath.build(SIGNAL__LINK, LINK__TARGET));
        row2.setJoinType(JoinType.INNER);
        row2.setOrigin(row0);

        final ObjectDataQuery row3 = TableFactory.eINSTANCE.createObjectDataQuery();
        row3.getObjects().add(ucaTable.getAnalysis());
        row3.setMetaclass(STAMPPackage.Literals.UCA_ANALYSIS);

        final DynamicDataset row = TableFactory.eINSTANCE.createDynamicDataset();
        row.getQueries().addAll(asList(row0, row1, row2, row3));

        final ObjectElement columnIdx = TableFactory.eINSTANCE.createObjectElement();
        columnIdx.setHeader(
                TableFactory.eINSTANCE.createFixedCellDescriptor(Messages.uca_table_no_label));
        final PseudoCellDescriptor columnIdxCell = TableFactory.eINSTANCE
                .createPseudoCellDescriptor();
        columnIdx.setCell(columnIdxCell);

        final ObjectElement columnCA = TableFactory.eINSTANCE.createObjectElement();
        columnCA.setHeader(
                TableFactory.eINSTANCE.createFixedCellDescriptor(Messages.uca_table_ca_label));
        final PropertyCellDescriptor columnCACell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        columnCACell.setOrigin(row.getQueries().indexOf(row0));
        applyFeaturePath(columnCACell, EFeaturePath.build(NAMED_ELEMENT__NAME));
        columnCACell.setLineWrap(true);
        columnCA.setCell(columnCACell);

        final ObjectElement columnFrom = TableFactory.eINSTANCE.createObjectElement();
        columnFrom.setHeader(
                TableFactory.eINSTANCE.createFixedCellDescriptor(Messages.uca_table_ca_from_label));
        final PropertyCellDescriptor columnFromCell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        columnFromCell.setOrigin(row.getQueries().indexOf(row1));
        applyFeaturePath(columnFromCell, EFeaturePath.build(NAMED_ELEMENT__NAME));
        columnFromCell.setLineWrap(true);
        columnFrom.setCell(columnFromCell);

        final ObjectElement columnTo = TableFactory.eINSTANCE.createObjectElement();
        columnTo.setHeader(
                TableFactory.eINSTANCE.createFixedCellDescriptor(Messages.uca_table_ca_to_label));
        final PropertyCellDescriptor columnToCell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        columnToCell.setOrigin(row.getQueries().indexOf(row2));
        applyFeaturePath(columnToCell, EFeaturePath.build(NAMED_ELEMENT__NAME));
        columnToCell.setLineWrap(true);
        columnTo.setCell(columnToCell);

        final ObjectElement columnCondition = TableFactory.eINSTANCE.createObjectElement();
        columnCondition.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.uca_table_ca_providing_condition_label));
        final PropertyCellDescriptor columnConditionCell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        columnConditionCell.setOrigin(row.getQueries().indexOf(row0));
        applyFeaturePath(columnConditionCell, EFeaturePath.build(SIGNAL__PROVIDING_CONDITION));
        columnConditionCell.setLineWrap(true);
        columnCondition.setCell(columnConditionCell);

        // ガイドワードの数分ヘッダを追加する
        final UCAAnalysis analysis = ucaTable.getAnalysis();
        final UCAGuideWordSet guideWordSet = analysis.getGuideWordSet();
        final ObjectDataQuery guideWordQuery = TableFactory.eINSTANCE.createObjectDataQuery();
        guideWordQuery.getObjects().add(guideWordSet);
        guideWordQuery.setMetaclass(STAMPPackage.Literals.UCA_GUIDE_WORD);
        guideWordQuery.setTraversal(true);

        final DynamicDataset rowGuideWord = TableFactory.eINSTANCE.createDynamicDataset();
        rowGuideWord.getQueries().addAll(asList(guideWordQuery));

        final PropertyCellDescriptor guideWordCellDescriptor = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        guideWordCellDescriptor.setOrigin(rowGuideWord.getQueries().indexOf(guideWordQuery));
        applyFeaturePath(guideWordCellDescriptor, EFeaturePath.build(NAMED_ELEMENT__NAME));
        guideWordCellDescriptor.setLineWrap(true);
        rowGuideWord.getHeaders().add(guideWordCellDescriptor);

        final NativeCellDescriptor ucaAnalysisResultCellDescriptor = TableFactory.eINSTANCE
                .createNativeCellDescriptor();
        ucaAnalysisResultCellDescriptor
                .setImplementationClass(UCAAnalysisResultCellEvaluator.class);
        rowGuideWord.getCells().add(ucaAnalysisResultCellDescriptor);

        final ElementSet columnSet = TableFactory.eINSTANCE.createElementSet();
        columnSet.getElements().addAll(
                asList(columnIdx, columnCA, columnFrom, columnTo, columnCondition));
        final FixedDataset column = TableFactory.eINSTANCE.createFixedDataset();
        column.getElementSets().add(columnSet);

        final Table table = TableFactory.eINSTANCE.createTable();
        table.getColumns().add(column);
        table.getColumns().add(rowGuideWord);
        table.getRows().add(row);

        return table;
    }

    @Override
    protected AdapterFactory createTableAdapterFactory() {
        final EditingDomain domain = TransactionUtil.getEditingDomain(getDiagram());
        return new GenericTableAdapterFactory(ItemProviders.getAdapterFactory(domain)) {
            @Override
            protected IItemProviderDecorator createGridDecorator() {
                return new GridItemProviderDecorator(this) {
                    @Override
                    protected Command createMoveRowElementCommand(final EObject owner,
                            final Tuple tuple,
                            final int index,
                            final EditingDomain defaultDomain) {
                        return createMoveControlActionCommand(owner, tuple, index, defaultDomain);
                    }
                };
            }

            protected Command createMoveControlActionCommand(final EObject owner,
                    final Tuple tuple,
                    final int index,
                    final EditingDomain defaultDomain) {
                final Tuple targetTuple = grid.getRowTuples().get(index);
                final ControlAction target = (ControlAction) targetTuple.getElementObject(0);
                final UCATable ucaTable = getUCATable();
                final UCAAnalysis ucaAnalysis = ucaTable != null ? ucaTable.getAnalysis() : null;
                if (ucaAnalysis == null) {
                    return IdentityCommand.INSTANCE;
                }
                final int targetIndex = ucaAnalysis.getControlActions().indexOf(target);
                final EObject value = tuple.getElementObject(0);
                return MoveCommand.create(getEditingDomainFor(ucaAnalysis), ucaAnalysis,
                        UCA_ANALYSIS__CONTROL_ACTION, value, targetIndex);
            }

            private UCATable getUCATable() {
                final Diagram diagram = getDiagram();
                if (diagram != null && diagram instanceof UCATable) {
                    return (UCATable) diagram;
                }
                return null;
            }
        };
    }

    @Override
    protected JTable createJTable(final AdapterFactory adapterFactory, final Grid grid) {
        final GenericTableColumnProvider columnProvider = new UcaTableColumnProvider();
        columnProvider.setAdapterFactory(adapterFactory);
        columnProvider.setGrid(grid);

        final TransactionalEditingDomain txDomain = TransactionUtil.getEditingDomain(getDiagram());
        return JTableBuilder.builder(txDomain)
                .adapterFactory(adapterFactory)
                .columnProvider(columnProvider)
                .transferHandler(new GenericTableTableTransferHandler(txDomain, domain))
                .input(grid)
                .font(resolveTableFont())
                .reorderingAllowed(false)
                .build(new GenericTable());
    }

    /**
     * コントロールアクション抽出用の{@link IQueryExecutor}実装です。
     */
    public static class ControlActionFactory implements IQueryExecutor.Provider {

        @Override
        public IQueryExecutor createExecutor(final DataQuery dataQuery) {
            return dependencies -> {
                final EAnnotation annotation = dataQuery.getEAnnotation(ANNOTATION__UCA_TABLE);
                final UCATable table = (UCATable) annotation.getReferences().get(0);
                return table.getControlActions().iterator();
            };
        }
    }

    @Override
    public ISelection getEditorSelection(final ITableSelection selection) {
        final IListSelection columnSelection = selection.getColumnSelection();
        final int[] columnIndices = columnSelection.getIndices();
        if (columnIndices.length > 0) {
            final List<Object> elements = new ArrayList<>(selection.size());
            for (final Tuple tuple : Selections.iterable(selection, Tuple.class)) {
                elements.add(tuple.getElementObject(0));
            }
            final IListSelection rowSelection = selection.getRowSelection();
            return new TableSelection(elements, rowSelection, columnSelection);
        }
        return super.getEditorSelection(selection);
    }

    @Override
    protected boolean needToShowMessageOnActivated() {
        // UCA表では操作ガイドは表示しない
        return false;
    }

    @Override
    protected String getMessageOnActivated() {
        // UCA表では操作ガイドは表示しない
        return null;
    }
}
